@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2013-2021 Laboratoire de Recherche et Développement
+// Copyright (C) 2013-2023 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -75,7 +75,7 @@
 #include <spot/twaalgos/sum.hh>
 #include <spot/twaalgos/totgba.hh>
 
-static const char argp_program_doc[] ="\
+static const char argp_program_doc[] = "\
 Convert, transform, and filter omega-automata.\v\
 Exit status:\n\
   0  if some automata were output\n\
@@ -88,6 +88,7 @@ enum {
   OPT_ACC_SETS,
   OPT_ACCEPT_WORD,
   OPT_ACCEPTANCE_IS,
+  OPT_ALIASES,
   OPT_AP_N,
   OPT_ARE_ISOMORPHIC,
   OPT_CLEAN_ACC,
@@ -158,6 +159,7 @@ enum {
   OPT_SUM_OR,
   OPT_SUM_AND,
   OPT_TERMINAL_SCCS,
+  OPT_TO_FINITE,
   OPT_TRIV_SCCS,
   OPT_USED_AP_N,
   OPT_UNUSED_AP_N,
@@ -175,6 +177,10 @@ static const argp_option options[] =
     /**************************************************/
     { "count", 'c', nullptr, 0, "print only a count of matched automata", 3 },
     { "max-count", 'n', "NUM", 0, "output at most NUM automata", 3 },
+    { "aliases", OPT_ALIASES, "drop|keep", 0,
+      "If the input automaton uses HOA aliases, this decides whether their "
+      "preservation should be attempted in the output.  The default is "
+      "\"keep\".", 3 },
     /**************************************************/
     { nullptr, 0, nullptr, 0, "Filtering options:", 5 },
     { "ap", OPT_AP_N, "RANGE", 0,
@@ -386,6 +392,10 @@ static const argp_option options[] =
       "solver can be set thanks to the SPOT_SATSOLVER environment variable"
       "(see spot-x)."
       , 0 },
+    { "to-finite", OPT_TO_FINITE, "alive", OPTION_ARG_OPTIONAL,
+      "Convert an automaton with \"alive\" and \"!alive\" propositions "
+      "into a Büchi automaton interpretable as a finite automaton.  "
+      "States with a outgoing \"!alive\" edge are marked as accepting.", 0 },
     { nullptr, 0, nullptr, 0, "Decorations (for -d and -H1.1 output):", 9 },
     { "highlight-accepting-run", OPT_HIGHLIGHT_ACCEPTING_RUN, "NUM",
       OPTION_ARG_OPTIONAL, "highlight one accepting run using color NUM", 0},
@@ -438,7 +448,7 @@ struct canon_aut
   std::vector<tr_t> edges;
   std::string acc;
 
-  canon_aut(const spot::const_twa_graph_ptr& aut)
+  explicit canon_aut(const spot::const_twa_graph_ptr& aut)
     : num_states(aut->num_states())
     , edges(aut->edge_vector().begin() + 1,
             aut->edge_vector().end())
@@ -474,6 +484,20 @@ static gra_type const gra_types[] =
 };
 ARGMATCH_VERIFY(gra_args, gra_types);
 
+static bool opt_aliases = true;
+static char const* const aliases_args[] =
+{
+  "drop", "no", "false", "0",
+  "keep", "yes", "true", "1",
+  nullptr,
+};
+static bool const aliases_types[] =
+{
+  false, false, false, false,
+  true, true, true, true,
+};
+ARGMATCH_VERIFY(aliases_args, aliases_types);
+
 enum acc_type {
   ACC_Any = 0,
   ACC_Given,
@@ -669,6 +693,7 @@ static bool opt_rem_unused_ap = false;
 static bool opt_sep_sets = false;
 static bool opt_split_edges = false;
 static const char* opt_sat_minimize = nullptr;
+static const char* opt_to_finite = nullptr;
 static int opt_highlight_nondet_states = -1;
 static int opt_highlight_nondet_edges = -1;
 static int opt_highlight_accepting_run = -1;
@@ -688,10 +713,12 @@ ensure_deterministic(const spot::twa_graph_ptr& aut, bool nonalt = false)
   return p.run(aut);
 }
 
-static spot::twa_graph_ptr ensure_tba(spot::twa_graph_ptr aut)
+static spot::twa_graph_ptr
+ensure_tba(spot::twa_graph_ptr aut,
+           spot::postprocessor::output_type type = spot::postprocessor::Buchi)
 {
   spot::postprocessor p;
-  p.set_type(spot::postprocessor::Buchi);
+  p.set_type(type);
   p.set_pref(spot::postprocessor::Any);
   p.set_level(spot::postprocessor::Low);
   return p.run(aut);
@@ -701,12 +728,14 @@ static spot::twa_graph_ptr ensure_tba(spot::twa_graph_ptr aut)
 static spot::twa_graph_ptr
 product(spot::twa_graph_ptr left, spot::twa_graph_ptr right)
 {
-  if ((type == spot::postprocessor::Buchi)
-      && (left->num_sets() + right->num_sets() >
-          spot::acc_cond::mark_t::max_accsets()))
+  // Are we likely to fail because of too many colors?
+  if ((left->num_sets() + right->num_sets() >
+       spot::acc_cond::mark_t::max_accsets())
+      && (type == spot::postprocessor::Buchi
+          || type == spot::postprocessor::CoBuchi))
     {
-      left = ensure_tba(left);
-      right = ensure_tba(right);
+      left = ensure_tba(left, type);
+      right = ensure_tba(right, type);
     }
   return spot::product(left, right);
 }
@@ -714,16 +743,34 @@ product(spot::twa_graph_ptr left, spot::twa_graph_ptr right)
 static spot::twa_graph_ptr
 product_or(spot::twa_graph_ptr left, spot::twa_graph_ptr right)
 {
-  if ((type == spot::postprocessor::Buchi)
-      && (left->num_sets() + right->num_sets() >
-          spot::acc_cond::mark_t::max_accsets()))
+  // Are we likely to fail because of too many colors?
+  if ((left->num_sets() + right->num_sets() >
+       spot::acc_cond::mark_t::max_accsets())
+      && (type == spot::postprocessor::Buchi
+          || type == spot::postprocessor::CoBuchi))
     {
-      left = ensure_tba(left);
-      right = ensure_tba(right);
+      left = ensure_tba(left, type);
+      right = ensure_tba(right, type);
     }
   return spot::product_or(left, right);
 }
 
+static spot::twa_graph_ptr
+word_to_aut(const char* word, const char *argname)
+{
+  try
+    {
+      return spot::parse_word(word, opt->dict)->as_automaton();
+    }
+  catch (const spot::parse_error& e)
+    {
+      error(2, 0, "failed to parse the argument of --%s:\n%s",
+            argname, e.what());
+    }
+  SPOT_UNREACHABLE();
+  return nullptr;
+}
+
 static int
 parse_opt(int key, char* arg, struct argp_state*)
 {
@@ -736,7 +783,7 @@ parse_opt(int key, char* arg, struct argp_state*)
       automaton_format = Count;
       break;
     case 'F':
-      jobs.emplace_back(arg, true);
+      jobs.emplace_back(arg, job_type::AUT_FILENAME);
       break;
     case 'n':
       opt_max_count = to_pos_int(arg, "-n/--max-count");
@@ -745,17 +792,17 @@ parse_opt(int key, char* arg, struct argp_state*)
       opt_nth = parse_range(arg, 0, std::numeric_limits<int>::max());
       break;
     case 'u':
-      opt->uniq = std::unique_ptr<unique_aut_t>(new std::set<canon_aut>());
+      opt->uniq = std::make_unique<unique_aut_t>();
       break;
     case 'v':
       opt_invert = true;
       break;
     case 'x':
-      {
-        const char* opt = extra_options.parse_options(arg);
-        if (opt)
-          error(2, 0, "failed to parse --options near '%s'", opt);
-      }
+      if (const char* opt = extra_options.parse_options(arg))
+        error(2, 0, "failed to parse --options near '%s'", opt);
+      break;
+    case OPT_ALIASES:
+      opt_aliases = XARGMATCH("--aliases", arg, aliases_args, aliases_types);
       break;
     case OPT_AP_N:
       opt_ap_n = parse_range(arg, 0, std::numeric_limits<int>::max());
@@ -768,16 +815,7 @@ parse_opt(int key, char* arg, struct argp_state*)
       opt_art_sccs_set = true;
       break;
     case OPT_ACCEPT_WORD:
-      try
-        {
-          opt->acc_words.push_back(spot::parse_word(arg, opt->dict)
-                                   ->as_automaton());
-        }
-      catch (const spot::parse_error& e)
-        {
-          error(2, 0, "failed to parse the argument of --accept-word:\n%s",
-                e.what());
-        }
+      opt->acc_words.emplace_back(word_to_aut(arg, "accept-word"));
       break;
     case OPT_ACCEPTANCE_IS:
       {
@@ -930,16 +968,7 @@ parse_opt(int key, char* arg, struct argp_state*)
                     "%d should be followed by a comma and WORD", res);
             arg = endptr + 1;
           }
-        try
-          {
-            opt->hl_words.emplace_back(spot::parse_word(arg, opt->dict)
-                                       ->as_automaton(), res);
-          }
-        catch (const spot::parse_error& e)
-          {
-            error(2, 0, "failed to parse the argument of --highlight-word:\n%s",
-                  e.what());
-          }
+        opt->hl_words.emplace_back(word_to_aut(arg, "highlight-word"), res);
       }
       break;
     case OPT_HIGHLIGHT_LANGUAGES:
@@ -960,7 +989,7 @@ parse_opt(int key, char* arg, struct argp_state*)
         if (!opt->included_in)
           opt->included_in = aut;
         else
-          opt->included_in = spot::product_or(opt->included_in, aut);
+          opt->included_in = ::product_or(opt->included_in, aut);
       }
       break;
     case OPT_INHERENTLY_WEAK_SCCS:
@@ -1123,16 +1152,7 @@ parse_opt(int key, char* arg, struct argp_state*)
       opt_art_sccs_set = true;
       break;
     case OPT_REJECT_WORD:
-      try
-        {
-          opt->rej_words.push_back(spot::parse_word(arg, opt->dict)
-                                   ->as_automaton());
-        }
-      catch (const spot::parse_error& e)
-        {
-          error(2, 0, "failed to parse the argument of --reject-word:\n%s",
-                e.what());
-        }
+      opt->rej_words.emplace_back(word_to_aut(arg, "reject-word"));
       break;
     case OPT_REM_AP:
       opt->rem_ap.add_ap(arg);
@@ -1203,6 +1223,9 @@ parse_opt(int key, char* arg, struct argp_state*)
       opt_terminal_sccs_set = true;
       opt_art_sccs_set = true;
       break;
+    case OPT_TO_FINITE:
+      opt_to_finite = arg ? arg : "alive";
+      break;
     case OPT_TRIV_SCCS:
       opt_triv_sccs = parse_range(arg, 0, std::numeric_limits<int>::max());
       opt_art_sccs_set = true;
@@ -1221,7 +1244,7 @@ parse_opt(int key, char* arg, struct argp_state*)
       opt_art_sccs_set = true;
       break;
     case ARGP_KEY_ARG:
-      jobs.emplace_back(arg, true);
+      jobs.emplace_back(arg, job_type::AUT_FILENAME);
       break;
 
     default:
@@ -1254,7 +1277,7 @@ namespace
   static
   bool match_acceptance(spot::twa_graph_ptr aut)
   {
-    auto& acc = aut->acc();
+    const spot::acc_cond& acc = aut->acc();
     switch (opt_acceptance_is)
       {
       case ACC_Any:
@@ -1309,8 +1332,7 @@ namespace
         {
           bool max;
           bool odd;
-          bool is_p = acc.is_parity(max, odd, true);
-          if (!is_p)
+          if (!acc.is_parity(max, odd, true))
             return false;
           switch (opt_acceptance_is)
             {
@@ -1363,6 +1385,8 @@ namespace
       spot::process_timer timer;
       timer.start();
 
+      auto aliases = get_aliases(haut->aut);
+
       // If --stats or --name is used, duplicate the automaton so we
       // never modify the original automaton (e.g. with
       // merge_edges()) and the statistics about it make sense.
@@ -1421,7 +1445,7 @@ namespace
       if (matched && opt_acceptance_is)
         matched = match_acceptance(aut);
 
-      if (matched && (opt_sccs_set | opt_art_sccs_set))
+      if (matched && (opt_sccs_set || opt_art_sccs_set))
         {
           spot::scc_info si(aut);
           unsigned n = si.scc_count();
@@ -1501,14 +1525,14 @@ namespace
           && spot::contains(aut, opt->equivalent_pos);
 
       if (matched && !opt->acc_words.empty())
-        for (auto& word_aut: opt->acc_words)
+        for (const spot::twa_graph_ptr& word_aut: opt->acc_words)
           if (spot::product(aut, word_aut)->is_empty())
             {
               matched = false;
               break;
             }
       if (matched && !opt->rej_words.empty())
-        for (auto& word_aut: opt->rej_words)
+        for (const spot::twa_graph_ptr& word_aut: opt->rej_words)
           if (!spot::product(aut, word_aut)->is_empty())
             {
               matched = false;
@@ -1624,6 +1648,9 @@ namespace
       if (opt_split_edges)
         aut = spot::split_edges(aut);
 
+      if (opt_to_finite)
+        aut = spot::to_finite(aut, opt_to_finite);
+
       if (randomize_st || randomize_tr)
         spot::randomize(aut, randomize_st, randomize_tr);
 
@@ -1639,14 +1666,9 @@ namespace
         aut->accepting_run()->highlight(opt_highlight_accepting_run);
 
       if (!opt->hl_words.empty())
-        for (auto& word_aut: opt->hl_words)
-          {
-            if (aut->acc().uses_fin_acceptance())
-              error(2, 0,
-                    "--highlight-word does not yet work with Fin acceptance");
-            if (auto run = spot::product(aut, word_aut.first)->accepting_run())
-              run->project(aut)->highlight(word_aut.second);
-          }
+        for (auto& [word_aut, color]: opt->hl_words)
+          if (auto run = spot::product(aut, word_aut)->accepting_run())
+            run->project(aut)->highlight(color);
 
       timer.stop();
       if (opt->uniq)
@@ -1660,6 +1682,13 @@ namespace
 
       ++match_count;
 
+      if (aliases)
+        {
+          if (opt_aliases)
+            set_aliases(aut, *aliases);
+          else
+            set_aliases(aut, {});
+        }
       printer.print(aut, timer, nullptr, haut->filename.c_str(), -1,
                     haut, prefix, suffix);
 
@@ -1714,15 +1743,17 @@ main(int argc, char** argv)
       post.set_level(level);
 
       autfilt_processor processor(post, o.dict);
-      if (processor.run())
-        return 2;
-
-      // Diagnose unused -x options
-      extra_options.report_unused_options();
+      int err = processor.run();
 
       if (automaton_format == Count)
         std::cout << match_count << std::endl;
 
+      // Diagnose unused -x options
+      if (!err)
+        extra_options.report_unused_options();
+      else
+        return 2;
+
       check_cout();
       return match_count ? 0 : 1;
     });
