@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012-2021 Laboratoire de Recherche et Développement
+// Copyright (C) 2012-2023 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -41,7 +41,7 @@
 #include <spot/twaalgos/isdet.hh>
 
 automaton_format_t automaton_format = Hoa;
-static const char* automaton_format_opt = nullptr;
+const char* automaton_format_opt = nullptr;
 const char* opt_name = nullptr;
 static const char* opt_output = nullptr;
 static const char* stats = "";
@@ -203,12 +203,18 @@ static const argp_option io_options[] =
       "to specify additional options as in --hoa=opt)", 0 },
     { "%M, %m", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
       "name of the automaton", 0 },
-    { "%S, %s", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
-      "number of reachable states", 0 },
-    { "%E, %e", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
-      "number of reachable edges", 0 },
-    { "%T, %t", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
-      "number of reachable transitions", 0 },
+    { "%S, %s, %[LETTER]S, %[LETTER]s",
+      0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
+      "number of states (add one LETTER to select (r) reachable [default], "
+      "(u) unreachable, (a) all).", 0 },
+    { "%E, %e, %[LETTER]E, %[LETTER]e",
+      0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
+      "number of edges (add one LETTER to select (r) reachable [default], "
+      "(u) unreachable, (a) all).", 0 },
+    { "%T, %t, %[LETTER]E, %[LETTER]e",
+      0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
+      "number of transitions (add one LETTER to select (r) reachable "
+      "[default], (u) unreachable, (a) all).", 0 },
     { "%A, %a", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
       "number of acceptance sets", 0 },
     { "%G, %g, %[LETTERS]G, %[LETTERS]g", 0, nullptr,
@@ -221,7 +227,7 @@ static const argp_option io_options[] =
       "(iw) inherently weak. Use uppercase letters to negate them.", 0 },
     { "%R, %[LETTERS]R", 0, nullptr,
       OPTION_DOC | OPTION_NO_USAGE,
-      "CPU time (excluding parsing), in seconds; Add LETTERS to restrict to"
+      "CPU time (excluding parsing), in seconds; Add LETTERS to restrict to "
       "(u) user time, (s) system time, (p) parent process, "
       "or (c) children processes.", 0 },
     { "%N, %n", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
@@ -268,12 +274,15 @@ static const argp_option o_options[] =
       "to specify additional options as in --hoa=opt)", 0 },
     { "%m", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
       "name of the automaton", 0 },
-    { "%s", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
-      "number of reachable states", 0 },
-    { "%e", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
-      "number of reachable edges", 0 },
-    { "%t", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
-      "number of reachable transitions", 0 },
+    { "%s, %[LETTER]s", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
+      "number of states (add one LETTER to select (r) reachable [default], "
+      "(u) unreachable, (a) all).", 0 },
+    { "%e, %[LETTER]e", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
+      "number of edges (add one LETTER to select (r) reachable [default], "
+      "(u) unreachable, (a) all).", 0 },
+    { "%t, %[LETTER]t", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
+      "number of transitions (add one LETTER to select (r) reachable "
+      "[default], (u) unreachable, (a) all).", 0 },
     { "%a", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
       "number of acceptance sets", 0 },
     { "%g, %[LETTERS]g", 0, nullptr,
@@ -444,7 +453,7 @@ hoa_stat_printer::print(const spot::const_parsed_aut_ptr& haut,
                         const spot::const_twa_graph_ptr& aut,
                         spot::formula f,
                         const char* filename, int loc,
-                        spot::process_timer& ptimer,
+                        const spot::process_timer& ptimer,
                         const char* csv_prefix, const char* csv_suffix)
 {
   timer_ = ptimer;
@@ -472,15 +481,15 @@ hoa_stat_printer::print(const spot::const_parsed_aut_ptr& haut,
       if (has('T'))
         {
           spot::twa_sub_statistics s = sub_stats_reachable(haut->aut);
-          haut_states_ = s.states;
-          haut_edges_ = s.edges;
-          haut_trans_ = s.transitions;
+          haut_states_.set(s.states, haut->aut->num_states());
+          haut_edges_.set(s.edges, haut->aut->num_edges());
+          haut_trans_.set(s.transitions, count_all_transitions(haut->aut));
         }
       else if (has('E') || has('S'))
         {
           spot::twa_statistics s = stats_reachable(haut->aut);
-          haut_states_ = s.states;
-          haut_edges_ = s.edges;
+          haut_states_.set(s.states, haut->aut->num_states());
+          haut_edges_.set(s.edges, haut->aut->num_edges());
         }
       if (has('M'))
         {
@@ -624,10 +633,10 @@ automaton_printer::print(const spot::twa_graph_ptr& aut,
       outputnamer.print(haut, aut, f, filename, loc, ptimer,
                         csv_prefix, csv_suffix);
       std::string fname = outputname.str();
-      auto p = outputfiles.emplace(fname, nullptr);
-      if (p.second)
-        p.first->second.reset(new output_file(fname.c_str()));
-      out = &p.first->second->ostream();
+      auto [it, b] = outputfiles.try_emplace(fname, nullptr);
+      if (b)
+        it->second.reset(new output_file(fname.c_str()));
+      out = &it->second->ostream();
     }
 
   // Output it.
