@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012-2022 Laboratoire de Recherche et Développement
+// Copyright (C) 2012-2023 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -20,13 +20,14 @@
 #include "common_setup.hh"
 #include "common_aoutput.hh"
 
-#include "argp.h"
-#include "closeout.h"
+#include <argp.h>
+#include <closeout.h>
 #include <cstdlib>
 #include <unistd.h>
 #include <iostream>
 #include <signal.h>
 #include <sys/wait.h>
+#include <error.h>
 #include <spot/misc/tmpfile.hh>
 
 static void
@@ -35,7 +36,7 @@ display_version(FILE *stream, struct argp_state*)
   fputs(program_name, stream);
   fputs(" (" PACKAGE_NAME ") " PACKAGE_VERSION "\n\
 \n\
-Copyright (C) 2022  Laboratoire de Recherche et Développement de l'Epita.\n\
+Copyright (C) 2023 Laboratoire de Recherche de l'Epita (LRE)\n\
 License GPLv3+: \
 GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.\n\
 This is free software: you are free to change and redistribute it.\n\
