@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2015-2021 Laboratoire de Recherche et Développement
+// Copyright (C) 2015-2023 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -27,7 +27,9 @@
 #include <sys/wait.h>
 #include <fcntl.h>
 #include <iomanip>
-#ifdef HAVE_SPAWN_H
+#include <fstream>
+#if __has_include(<spawn.h>)
+#define HAVE_SPAWN_H 1
 #include <spawn.h>
 #endif
 #include <regex>
@@ -51,7 +53,7 @@ struct shorthands_t
 };
 #define SHORTHAND(PRE, POST) { PRE, std::regex("^" PRE), POST }
 
-static shorthands_t shorthands_ltl[] = {
+static const shorthands_t shorthands_ltl[] = {
   SHORTHAND("delag", " %f>%O"),
   SHORTHAND("lbt", " <%L>%O"),
   SHORTHAND("ltl2ba", " -f %s>%O"),
@@ -71,7 +73,7 @@ static shorthands_t shorthands_ltl[] = {
   SHORTHAND("owl.* ltl-utilities\\b", " -f %f"),
 };
 
-static shorthands_t shorthands_autproc[] = {
+static const shorthands_t shorthands_autproc[] = {
   SHORTHAND("autfilt", " %H>%O"),
   SHORTHAND("dra2dpa", " <%H>%O"),
   SHORTHAND("dstar2tgba", " %H>%O"),
@@ -83,7 +85,7 @@ static shorthands_t shorthands_autproc[] = {
             " <%H>%O"),
 };
 
-static void show_shorthands(shorthands_t* begin, shorthands_t* end)
+static void show_shorthands(const shorthands_t* begin, const shorthands_t* end)
 {
   std::cout
     << ("If a COMMANDFMT does not use any %-sequence, and starts with one of\n"
@@ -98,7 +100,8 @@ static void show_shorthands(shorthands_t* begin, shorthands_t* end)
 }
 
 
-tool_spec::tool_spec(const char* spec, shorthands_t* begin, shorthands_t* end,
+tool_spec::tool_spec(const char* spec,
+                     const shorthands_t* begin, const shorthands_t* end,
                      bool is_ref) noexcept
   : spec(spec), cmd(spec), name(spec), reference(is_ref)
 {
@@ -111,15 +114,15 @@ tool_spec::tool_spec(const char* spec, shorthands_t* begin, shorthands_t* end,
         {
           if (*pos == '{')
             ++count;
-          else if (*pos == '}')
-            if (!--count)
-              {
-                name = strndup(cmd + 1, pos - cmd - 1);
-                cmd = pos + 1;
-                while (*cmd == ' ' || *cmd == '\t')
-                  ++cmd;
-                break;
-              }
+          else if (*pos == '}' && --count == 0)
+            {
+              name = strndup(cmd + 1, pos - cmd - 1);
+              cmd = pos + 1;
+              // skip leading whitespace
+              while (*cmd == ' ' || *cmd == '\t')
+                ++cmd;
+              break;
+            }
         }
     }
   // If there is no % in the string, look for a known
@@ -145,11 +148,11 @@ tool_spec::tool_spec(const char* spec, shorthands_t* begin, shorthands_t* end,
           auto& p = *begin++;
           if (std::regex_search(basename, p.rprefix))
             {
-              int m = strlen(p.suffix);
-              int q = strlen(cmd);
+              size_t m = strlen(p.suffix);
+              size_t q = strlen(cmd);
               char* tmp = static_cast<char*>(malloc(q + m + 1));
-              strcpy(tmp, cmd);
-              strcpy(tmp + q, p.suffix);
+              memcpy(tmp, cmd, q);
+              memcpy(tmp + q, p.suffix, m + 1);
               cmd = tmp;
               allocated = true;
               break;
@@ -460,6 +463,92 @@ autproc_runner::round_automaton(spot::const_twa_graph_ptr aut, unsigned serial)
   filename_automaton.new_round(aut, serial);
 }
 
+
+std::string
+read_stdout_of_command(char* const* args)
+{
+#if HAVE_SPAWN_H
+  int cout_pipe[2];
+  if (int err = pipe(cout_pipe))
+    error(2, err, "pipe() failed");
+
+  posix_spawn_file_actions_t actions;
+  if (int err = posix_spawn_file_actions_init(&actions))
+    error(2, err, "posix_spawn_file_actions_init() failed");
+
+  posix_spawn_file_actions_addclose(&actions, STDIN_FILENO);
+  posix_spawn_file_actions_addclose(&actions, cout_pipe[0]);
+  posix_spawn_file_actions_adddup2(&actions, cout_pipe[1], STDOUT_FILENO);
+  posix_spawn_file_actions_addclose(&actions, cout_pipe[1]);
+
+  pid_t pid;
+  if (int err = posix_spawnp(&pid, args[0], &actions, nullptr, args, environ))
+    error(2, err, "failed to run '%s'", args[0]);
+
+  if (int err = posix_spawn_file_actions_destroy(&actions))
+    error(2, err, "posix_spawn_file_actions_destroy() failed");
+
+  if (close(cout_pipe[1]) < 0)
+    error(2, errno, "closing write-side of pipe failed");
+
+  std::string results;
+  ssize_t bytes_read;
+  for (;;)
+    {
+      static char buffer[512];
+      bytes_read = read(cout_pipe[0], buffer, sizeof(buffer));
+      if (bytes_read > 0)
+        results.insert(results.end(), buffer, buffer + bytes_read);
+      else
+        break;
+    }
+  if (bytes_read < 0)
+    error(2, bytes_read, "failed to read from pipe");
+
+  if (cout_pipe[0] < 0)
+    error(2, errno, "closing read-side of pipe failed");
+
+  int exit_code = 0;
+  if (waitpid(pid, &exit_code, 0) == -1)
+    error(2, errno, "waitpid() failed");
+
+  if (exit_code)
+    error(2, 0, "'%s' exited with status %d", args[0], exit_code);
+
+  return results;
+#else
+  // We could provide a pipe+fork+exec alternative implementation, but
+  // systems without posix_spawn() might also not have fork and exec.
+  // For instance MinGW does not.  So let's fallback to system+tmpfile
+  // instead for maximum portability.
+  char prefix[30];
+  snprintf(prefix, sizeof prefix, "spot-tmp");
+  spot::temporary_file* tmpfile = spot::create_tmpfile(prefix);
+  std::string tmpname = tmpfile->name();
+  std::ostringstream cmd;
+  for (auto t = args; *t != nullptr; ++t)
+    spot::quote_shell_string(cmd, *t) << ' ';
+  cmd << '>';
+  spot::quote_shell_string(cmd, tmpfile->name());
+  std::string cmdstr = cmd.str();
+  int exit_code = system(cmdstr.c_str());
+  if (exit_code < 0)
+    error(2, errno, "failed to execute %s", cmdstr.c_str());
+  if (exit_code > 0)
+    error(2, 0, "'%s' exited with status %d", args[0], exit_code);
+
+  std::ifstream ifs(tmpname,  std::ifstream::in);
+  if (!ifs)
+    error(2, 0, "failed to open %s (output of %s)", tmpname.c_str(), args[0]);
+  ifs.exceptions(std::ifstream::failbit | std::ifstream::badbit);
+  std::stringstream buffer;
+  buffer << ifs.rdbuf();
+  delete tmpfile;
+  return buffer.str();
+#endif
+}
+
+
 std::atomic<bool> timed_out{false};
 unsigned timeout_count = 0;
 
@@ -523,7 +612,7 @@ get_arg(const char*& cmd)
 {
   const char* start = cmd;
   std::string arg;
-  while (int c = *cmd)
+  while (char c = *cmd)
     {
       switch (c)
         {
@@ -553,14 +642,14 @@ get_arg(const char*& cmd)
           goto end_loop;
         case '\'':
           {
-            int d = 0;
+            char d = '\0';
             while ((d = *++cmd))
               {
                 if (d == '\'')
                   break;
                 arg.push_back(d);
               }
-            if (d == 0)
+            if (d == '\0')
               return nullptr;
           }
           break;
@@ -705,6 +794,7 @@ parse_simple_command(const char* cmd)
   return res;
 }
 
+
 #ifndef HAVE_SPAWN_H
 static void
 exec_command(const char* cmd)
@@ -750,8 +840,6 @@ exec_command(const char* cmd)
   SPOT_UNREACHABLE();
   return;
 }
-#else
-extern char **environ;
 #endif
 
 int
