@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2013-2019 Laboratoire de Recherche et Développement
+// Copyright (C) 2013-2019, 2022, 2023 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -48,7 +48,7 @@
 #include <spot/parseaut/public.hh>
 #include <spot/twaalgos/sccinfo.hh>
 
-static const char argp_program_doc[] ="\
+static const char argp_program_doc[] = "\
 Convert automata with any acceptance condition into variants of \
 Büchi automata.\n\nThis reads automata into any supported format \
 (HOA, LBTT, ltl2dstar, never claim) and outputs a \
@@ -89,7 +89,7 @@ parse_opt(int key, char* arg, struct argp_state*)
   switch (key)
     {
     case 'F':
-      jobs.emplace_back(arg, true);
+      jobs.emplace_back(arg, job_type::AUT_FILENAME);
       break;
     case 'x':
       {
@@ -99,7 +99,7 @@ parse_opt(int key, char* arg, struct argp_state*)
       }
       break;
     case ARGP_KEY_ARG:
-      jobs.emplace_back(arg, true);
+      jobs.emplace_back(arg, job_type::AUT_FILENAME);
       break;
     default:
       return ARGP_ERR_UNKNOWN;
@@ -117,7 +117,7 @@ namespace
     spot::postprocessor& post;
     automaton_printer printer;
 
-    dstar_processor(spot::postprocessor& post)
+    explicit dstar_processor(spot::postprocessor& post)
       : hoa_processor(spot::make_bdd_dict()), post(post), printer(aut_input)
     {
     }
