@@ -1,6 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2017-2019 Laboratoire de Recherche et Développement
-// de l'Epita (LRDE).
+// Copyright (C) 2017-2019, 2022-2023 Laboratoire de Recherche et
+// Développement de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
 //
@@ -43,7 +43,8 @@
 
 using namespace spot;
 
-const char argp_program_doc[] ="Generate ω-automata from predefined patterns.";
+static const char argp_program_doc[] =
+  "Generate ω-automata from predefined patterns.";
 
 static const argp_option options[] =
   {
@@ -62,6 +63,11 @@ static const argp_option options[] =
     { "m-nba", gen::AUT_M_NBA, "RANGE", 0,
       "An NBA with N+1 states whose determinization needs at least "
       "N! states", 0},
+    { "cyclist-trace-nba", gen::AUT_CYCLIST_TRACE_NBA, "RANGE", 0,
+      "An NBA with N+2 states that should include cyclist-proof-dba=B.", 0},
+    { "cyclist-proof-dba", gen::AUT_CYCLIST_PROOF_DBA, "RANGE", 0,
+      "A DBA with N+2 states that should be included "
+      "in cyclist-trace-nba=B.", 0},
     RANGE_DOC,
   /**************************************************/
     { nullptr, 0, nullptr, 0, "Miscellaneous options:", -1 },
@@ -122,7 +128,7 @@ output_pattern(gen::aut_pattern_id pattern, int n)
 static void
 run_jobs()
 {
-  for (auto& j: jobs)
+  for (const auto& j: jobs)
     {
       int inc = (j.range.max < j.range.min) ? -1 : 1;
       int n = j.range.min;
