@@ -1,6 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012, 2013, 2015-2019 Laboratoire de Recherche et
-// Développement de l'Epita (LRDE).
+// Copyright (C) 2012, 2013, 2015-2019, 2022-2023 Laboratoire de
+// Recherche et Développement de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
 //
@@ -45,7 +45,7 @@
 
 using namespace spot;
 
-const char argp_program_doc[] ="\
+static const char argp_program_doc[] = "\
 Generate temporal logic formulas from predefined patterns.";
 
 // We reuse the values from gen::ltl_pattern_id as option keys.
@@ -84,6 +84,8 @@ static const argp_option options[] =
     { "eh-patterns", gen::LTL_EH_PATTERNS, "RANGE", OPTION_ARG_OPTIONAL,
       "Etessami and Holzmann [Concur'00] patterns "
       "(range should be included in 1..12)", 0 },
+    { "eil-gsi", gen::LTL_EIL_GSI, "RANGE", 0,
+      "G[0..n]((a S b) -> c) rewritten using future operators", 0 },
     { "fxg-or", gen::LTL_FXG_OR, "RANGE", 0,
       "F(p0 | XG(p1 | XG(p2 | ... XG(pn))))", 0},
     { "gf-equiv", gen::LTL_GF_EQUIV, "RANGE", 0,
@@ -315,7 +317,7 @@ output_pattern(gen::ltl_pattern_id pattern, int n, int n2)
 static void
 run_jobs()
 {
-  for (auto& j: jobs)
+  for (const auto& j: jobs)
     {
       int inc = (j.range.max < j.range.min) ? -1 : 1;
       int n = j.range.min;
