@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012-2021 Laboratoire de Recherche et Développement
+// Copyright (C) 2012-2023 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -38,6 +38,7 @@
 #include <spot/misc/hash.hh>
 #include <spot/misc/timer.hh>
 #include <spot/tl/simplify.hh>
+#include <spot/tl/sonf.hh>
 #include <spot/tl/length.hh>
 #include <spot/tl/relabel.hh>
 #include <spot/tl/unabbrev.hh>
@@ -58,7 +59,7 @@
 #include <spot/twaalgos/totgba.hh>
 #include <spot/twaalgos/word.hh>
 
-const char argp_program_doc[] ="\
+static const char argp_program_doc[] = "\
 Read a list of formulas and output them back after some optional processing.\v\
 Exit status:\n\
   0  if some formulas were output (skipped syntax errors do not count)\n\
@@ -100,6 +101,8 @@ enum {
   OPT_SIZE_MAX,
   OPT_SIZE_MIN,
   OPT_SKIP_ERRORS,
+  OPT_SONF,
+  OPT_SONF_APS,
   OPT_STUTTER_INSENSITIVE,
   OPT_SUSPENDABLE,
   OPT_SYNTACTIC_GUARANTEE,
@@ -127,6 +130,11 @@ static const argp_option options[] =
     { "negate", OPT_NEGATE, nullptr, 0, "negate each formula", 0 },
     { "nnf", OPT_NNF, nullptr, 0,
       "rewrite formulas in negative normal form", 0 },
+    { "sonf", OPT_SONF, "PREFIX", OPTION_ARG_OPTIONAL,
+      "rewrite formulas in suffix operator normal form", 0 },
+    { "sonf-aps", OPT_SONF_APS, "FILENAME", OPTION_ARG_OPTIONAL,
+      "when used with --sonf, output the newly introduced atomic "
+      "propositions", 0 },
     { "relabel", OPT_RELABEL, "abc|pnn", OPTION_ARG_OPTIONAL,
       "relabel all atomic propositions, alphabetically unless " \
       "specified otherwise", 0 },
@@ -316,6 +324,7 @@ static range opt_nth = { 0, std::numeric_limits<int>::max() };
 static int opt_max_count = -1;
 static long int match_count = 0;
 static const char* from_ltlf = nullptr;
+static const char* sonf = nullptr;
 
 
 // We want all these variables to be destroyed when we exit main, to
@@ -327,6 +336,7 @@ static struct opt_t
   spot::bdd_dict_ptr dict = spot::make_bdd_dict();
   spot::exclusive_ap excl_ap;
   std::unique_ptr<output_file> output_define = nullptr;
+  std::unique_ptr<output_file> output_sonf = nullptr;
   spot::formula implied_by = nullptr;
   spot::formula imply = nullptr;
   spot::formula equivalent_to = nullptr;
@@ -377,7 +387,7 @@ parse_opt(int key, char* arg, struct argp_state*)
       break;
     case ARGP_KEY_ARG:
       // FIXME: use stat() to distinguish filename from string?
-      jobs.emplace_back(arg, true);
+      jobs.emplace_back(arg, job_type::LTL_FILENAME);
       break;
     case OPT_ACCEPT_WORD:
       try
@@ -460,6 +470,12 @@ parse_opt(int key, char* arg, struct argp_state*)
     case OPT_NNF:
       nnf = true;
       break;
+    case OPT_SONF:
+      sonf = arg ? arg : "sonf_";
+      break;
+    case OPT_SONF_APS:
+      opt->output_sonf.reset(new output_file(arg ? arg : "-"));
+      break;
     case OPT_OBLIGATION:
       obligation = true;
       break;
@@ -570,7 +586,7 @@ namespace
     fset_t unique_set;
     spot::relabeling_map relmap;
 
-    ltl_processor(spot::tl_simplifier& simpl)
+    explicit ltl_processor(spot::tl_simplifier& simpl)
       : simpl(simpl)
     {
     }
@@ -650,6 +666,25 @@ namespace
       if (nnf)
         f = simpl.negative_normal_form(f);
 
+      if (sonf != nullptr)
+        {
+          std::vector<std::string> new_aps;
+          std::tie(f, new_aps) = suffix_operator_normal_form(f, sonf);
+
+          if (opt->output_sonf
+              && output_format != count_output
+              && output_format != quiet_output)
+            {
+              for (size_t i = 0; i < new_aps.size(); ++i)
+                {
+                  if (i > 0)
+                    opt->output_sonf->ostream() << ' ';
+                  opt->output_sonf->ostream() << new_aps[i];
+                }
+              opt->output_sonf->ostream() << '\n';
+            }
+        }
+
       switch (relabeling)
         {
         case ApRelabeling:
@@ -687,7 +722,7 @@ namespace
       matched &= !syntactic_si || f.is_syntactic_stutter_invariant();
       if (matched && (ap_n.min > 0 || ap_n.max >= 0))
         {
-          auto s = atomic_prop_collect(f);
+          spot::atomic_prop_set* s = atomic_prop_collect(f);
           int n = s->size();
           delete s;
           matched &= (ap_n.min <= 0) || (n >= ap_n.min);
@@ -726,7 +761,7 @@ namespace
               aut = ltl_to_tgba_fm(f, simpl.get_dict(), true);
 
               if (matched && !opt->acc_words.empty())
-                for (auto& word_aut: opt->acc_words)
+                for (const spot::twa_graph_ptr& word_aut: opt->acc_words)
                   if (spot::product(aut, word_aut)->is_empty())
                     {
                       matched = false;
@@ -734,7 +769,7 @@ namespace
                     }
 
               if (matched && !opt->rej_words.empty())
-                for (auto& word_aut: opt->rej_words)
+                for (const spot::twa_graph_ptr& word_aut: opt->rej_words)
                   if (!spot::product(aut, word_aut)->is_empty())
                     {
                       matched = false;
@@ -808,12 +843,12 @@ namespace
             {
               // Sort the formulas alphabetically.
               std::map<std::string, spot::formula> m;
-              for (auto& p: relmap)
-                m.emplace(str_psl(p.first), p.second);
-              for (auto& p: m)
+              for (const auto& [newformula, oldname]: relmap)
+                m.emplace(str_psl(newformula), oldname);
+              for (const auto& [newname, oldname]: m)
                 stream_formula(opt->output_define->ostream()
-                               << "#define " << p.first << " (",
-                               p.second, filename,
+                               << "#define " << newname << " (",
+                               oldname, filename,
                                std::to_string(linenum).c_str()) << ")\n";
             }
           one_match = true;
@@ -841,7 +876,7 @@ main(int argc, char** argv)
         exit(err);
 
       if (jobs.empty())
-        jobs.emplace_back("-", 1);
+        jobs.emplace_back("-", job_type::LTL_FILENAME);
 
       if (boolean_to_isop && simplification_level == 0)
         simplification_level = 1;
