@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012-2019 Laboratoire de Recherche et Developpement
+// Copyright (C) 2012-2019, 2022 Laboratoire de Recherche et Developpement
 // de l'EPITA (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -1198,13 +1198,13 @@ namespace spot
     }
 
     static formula
-    pps_arbiter(std::string r_, std::string g_, int n, bool strict_)
+    pps_arbiter(std::string r_, std::string g_, unsigned n, bool strict_)
     {
       formula* r = new formula[n];
       formula* g = new formula[n];
       std::vector<formula> res;
 
-      for (int i = 0; i < n; ++i)
+      for (unsigned i = 0; i < n; ++i)
         {
           r[i] = formula::ap(r_ + std::to_string(i + 1));
           g[i] = formula::ap(g_ + std::to_string(i + 1));
@@ -1218,17 +1218,17 @@ namespace spot
       formula phi_s;
       {
         std::vector<formula> res;
-        for (int i = 0; i < n; ++i)
+        for (unsigned i = 0; i < n; ++i)
           res.push_back(formula::Not(r[i]));
         theta_e = formula::And(res);
 
         res.clear();
-        for (int i = 0; i < n; ++i)
+        for (unsigned i = 0; i < n; ++i)
           res.push_back(formula::Not(g[i]));
         theta_s = formula::And(res);
 
         res.clear();
-        for (int i = 0; i < n; ++i)
+        for (unsigned i = 0; i < n; ++i)
           {
             formula left = formula::Xor(r[i], g[i]);
             formula right = formula::Equiv(r[i], formula::X(r[i]));
@@ -1237,9 +1237,9 @@ namespace spot
         psi_e = formula::And(res);
 
         res.clear();
-        for (int i = 0; i < n; ++i)
+        for (unsigned i = 0; i < n; ++i)
           {
-            for (int j = 0; j < i; ++j)
+            for (unsigned j = 0; j < i; ++j)
               res.push_back(formula::Not(formula::And({g[i], g[j]})));
             formula left = formula::Equiv(r[i], g[i]);
             formula right = formula::Equiv(g[i], formula::X(g[i]));
@@ -1248,7 +1248,7 @@ namespace spot
         psi_s = formula::And(res);
 
         res.clear();
-        for (int i = 0; i < n; ++i)
+        for (unsigned i = 0; i < n; ++i)
           {
             formula f = formula::Not(formula::And({r[i], g[i]}));
             res.push_back(formula::G(formula::F(f)));
@@ -1256,7 +1256,7 @@ namespace spot
         phi_e = formula::And(res);
 
         res.clear();
-        for (int i = 0; i < n; ++i)
+        for (unsigned i = 0; i < n; ++i)
           {
             res.push_back(formula::G(formula::F(formula::Equiv(r[i], g[i]))));
           }
@@ -1267,9 +1267,9 @@ namespace spot
 
       if (!strict_)
         {
+          formula left = formula::And({formula::G(psi_e), phi_e});
           formula imp =
-            formula::Implies(formula::And({formula::G(psi_e), phi_e}),
-                             formula::And({formula::G(psi_s), phi_s}));
+            formula::Implies(left, formula::And({formula::G(psi_s), phi_s}));
           return formula::Implies(theta_e, formula::And({theta_s, imp}));
         }
       else
@@ -1281,6 +1281,21 @@ namespace spot
         }
     }
 
+    // G[0..n]((a S b) -> c) rewritten using future operators,
+    // from Edmond Irani Liu (EIL).  GSI stands for "Globally Since Implies."
+    static formula eil_gsi(int n, std::string a, std::string b, std::string c)
+    {
+      formula fa = formula::ap(a);
+      formula fb = formula::ap(b);
+      formula res = fb;
+      for (int i = 1; i <= n; ++i)
+        {
+          formula tmp = formula::And({formula::strong_X(i, fa), res});
+          res = formula::Or({formula::strong_X(i, fb), tmp});
+        }
+      return formula::Implies(res, formula::strong_X(n, formula::ap(c)));
+    }
+
     formula ltl_pattern(ltl_pattern_id pattern, int n, int m)
     {
       if (n < 0)
@@ -1317,6 +1332,8 @@ namespace spot
           return dac_pattern(n);
         case LTL_EH_PATTERNS:
           return eh_pattern(n);
+        case LTL_EIL_GSI:
+          return eil_gsi(n, "a", "b", "c");
         case LTL_FXG_OR:
           return FXG_or_n("p", n);
         case LTL_GF_EQUIV:
@@ -1418,6 +1435,7 @@ namespace spot
           "ccj-beta-prime",
           "dac-patterns",
           "eh-patterns",
+          "eil-gsi",
           "fxg-or",
           "gf-equiv",
           "gf-equiv-xn",
@@ -1485,6 +1503,7 @@ namespace spot
           return 55;
         case LTL_EH_PATTERNS:
           return 12;
+        case LTL_EIL_GSI:
         case LTL_FXG_OR:
         case LTL_GF_EQUIV:
         case LTL_GF_EQUIV_XN:
@@ -1554,6 +1573,7 @@ namespace spot
         case LTL_CCJ_BETA_PRIME:
         case LTL_DAC_PATTERNS:
         case LTL_EH_PATTERNS:
+        case LTL_EIL_GSI:
         case LTL_FXG_OR:
         case LTL_GF_EQUIV:
         case LTL_GF_EQUIV_XN:
