@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2011, 2012, 2014-2020 Laboratoire de
+// Copyright (C) 2011, 2012, 2014-2019 Laboratoire de
 // Recherche et Développement de l'Epita (LRDE)
 //
 // This file is part of Spot, a model checking library.
@@ -26,11 +26,6 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
-// MinGW does not define this.
-#ifndef WEXITSTATUS
-# define WEXITSTATUS(x) ((x) & 0xff)
-#endif
-
 #include <spot/ltsmin/ltsmin.hh>
 #include <spot/misc/hashfunc.hh>
 #include <spot/misc/fixpool.hh>
@@ -38,58 +33,22 @@
 #include <spot/misc/intvcomp.hh>
 #include <spot/misc/intvcmp2.hh>
 
+#include <spot/twaalgos/reachiter.hh>
+#include <string.h>
+#include <spot/mc/utils.hh>
+
 using namespace std::string_literals;
 
 namespace spot
 {
   namespace
   {
-
-    ////////////////////////////////////////////////////////////////////////
-    // spins interface
-
-    typedef struct transition_info {
-      int* labels; // edge labels, NULL, or pointer to the edge label(s)
-      int  group;  // holds transition group or -1 if unknown
-    } transition_info_t;
-
-    typedef void (*TransitionCB)(void *ctx,
-                                 transition_info_t *transition_info,
-                                 int *dst);
-  }
-
-  struct spins_interface
-  {
-    lt_dlhandle handle;        // handle to the dynamic library
-    void (*get_initial_state)(void *to);
-    int (*have_property)();
-    int (*get_successors)(void* m, int *in, TransitionCB, void *arg);
-    int (*get_state_size)();
-    const char* (*get_state_variable_name)(int var);
-    int (*get_state_variable_type)(int var);
-    int (*get_type_count)();
-    const char* (*get_type_name)(int type);
-    int (*get_type_value_count)(int type);
-    const char* (*get_type_value_name)(int type, int value);
-
-    ~spins_interface()
-    {
-      if (handle)
-        lt_dlclose(handle);
-      lt_dlexit();
-    }
-  };
-
-  namespace
-  {
-    typedef std::shared_ptr<const spins_interface> spins_interface_ptr;
-
     ////////////////////////////////////////////////////////////////////////
     // STATE
 
     struct spins_state final: public state
     {
-      spins_state(int s, fixed_size_pool* p)
+      spins_state(int s, fixed_size_pool<pool_type::Safe>* p)
         : pool(p), size(s), count(1)
       {
       }
@@ -138,7 +97,7 @@ namespace spot
       }
 
     public:
-      fixed_size_pool* pool;
+      fixed_size_pool<pool_type::Safe>* pool;
       size_t hash_value: 32;
       int size: 16;
       mutable unsigned count: 16;
@@ -233,7 +192,8 @@ namespace spot
     void transition_callback(void* arg, transition_info_t*, int *dst)
     {
       callback_context* ctx = static_cast<callback_context*>(arg);
-      fixed_size_pool* p = static_cast<fixed_size_pool*>(ctx->pool);
+      fixed_size_pool<pool_type::Safe>* p =
+        static_cast<fixed_size_pool<pool_type::Safe>*>(ctx->pool);
       spins_state* out =
         new(p->allocate()) spins_state(ctx->state_size, p);
       SPOT_ASSUME(out != nullptr);
@@ -722,7 +682,8 @@ namespace spot
           }
         else
           {
-            fixed_size_pool* p = const_cast<fixed_size_pool*>(&statepool_);
+            fixed_size_pool<pool_type::Safe>* p =
+              const_cast<fixed_size_pool<pool_type::Safe>*>(&statepool_);
             spins_state* res = new(p->allocate()) spins_state(state_size_, p);
             SPOT_ASSUME(res != nullptr);
             d_->get_initial_state(res->vars);
@@ -931,7 +892,7 @@ namespace spot
       void (*decompress_)(const int*, size_t, int*, size_t);
       int* uncompressed_;
       int* compressed_;
-      fixed_size_pool statepool_;
+      fixed_size_pool<pool_type::Safe> statepool_;
       multiple_size_pool compstatepool_;
 
       // This cache is used to speedup repeated calls to state_condition()
@@ -943,152 +904,49 @@ namespace spot
       mutable bdd state_condition_last_cond_;
       mutable callback_context* state_condition_last_cc_;
     };
-
-
-    //////////////////////////////////////////////////////////////////////////
-    // LOADER
-
-
-    // Call spins to compile "foo.prom" as "foo.prom.spins" if the latter
-    // does not exist already or is older.
-    static void
-    compile_model(std::string& filename, const std::string& ext)
-    {
-      std::string command;
-      std::string compiled_ext;
-
-      if (ext == ".gal")
-        {
-          command = "gal2c " + filename;
-          compiled_ext = "2C";
-        }
-      else if (ext == ".prom" || ext == ".pm" || ext == ".pml")
-        {
-          command = "spins " + filename;
-          compiled_ext = ".spins";
-        }
-      else if (ext == ".dve")
-        {
-          command = "divine compile --ltsmin " + filename;
-          compiled_ext = "2C";
-        }
-      else
-        {
-          throw std::runtime_error("Unknown extension '"s + ext +
-                                   "'.  Use '.prom', '.pm', '.pml', "
-                                   "'.dve', '.dve2C', '.gal', '.gal2C' or "
-                                   "'.prom.spins'.");
-        }
-
-      struct stat s;
-      if (stat(filename.c_str(), &s) != 0)
-        throw std::runtime_error("Cannot open "s + filename);
-
-      filename += compiled_ext;
-
-      // Remove any directory, because the new file will
-      // be compiled in the current directory.
-      size_t pos = filename.find_last_of("/\\");
-      if (pos != std::string::npos)
-        filename = "./" + filename.substr(pos + 1);
-
-      struct stat d;
-      if (stat(filename.c_str(), &d) == 0)
-        if (s.st_mtime < d.st_mtime)
-          // The .spins or .dve2C or .gal2C is up-to-date, no need to compile.
-          return;
-
-      int res = system(command.c_str());
-      if (res)
-        throw std::runtime_error("Execution of '"s
-                                 + command.c_str() + "' returned exit code "
-                                 + std::to_string(WEXITSTATUS(res)));
-    }
-
   }
 
   ltsmin_model
   ltsmin_model::load(const std::string& file_arg)
   {
-    std::string file;
-    if (file_arg.find_first_of("/\\") != std::string::npos)
-      file = file_arg;
-    else
-      file = "./" + file_arg;
-
-    std::string ext = file.substr(file.find_last_of("."));
-    if (ext != ".spins" && ext != ".dve2C" && ext != ".gal2C")
-      {
-        compile_model(file, ext);
-        ext = file.substr(file.find_last_of("."));
-      }
-
-    if (lt_dlinit())
-      throw std::runtime_error("Failed to initialize libltldl.");
-
-    lt_dlhandle h = lt_dlopen(file.c_str());
-    if (!h)
-      {
-        std::string lt_error = lt_dlerror();
-        lt_dlexit();
-        throw std::runtime_error("Failed to load '"s
-                                 + file + "'.\n" + lt_error);
-      }
-
-    auto d = std::make_shared<spins_interface>();
-    assert(d); // Superfluous, but Debian's GCC 7 snapshot 20161207-1 warns
-               // about potential null pointer dereference on the next line.
-    d->handle = h;
-
-
-    auto sym = [&](auto* dst, const char* name)
-      {
-        // Work around -Wpendantic complaining that pointer-to-objects
-        // should not be converted to pointer-to-functions (we have to
-        // assume they can for POSIX).
-        *reinterpret_cast<void**>(dst) = lt_dlsym(h, name);
-        if (*dst == nullptr)
-          throw std::runtime_error("Failed to resolve symbol '"s
-                                   + name + "' in '" + file + "'.");
-      };
-
-    // SpinS interface.
-    if (ext == ".spins")
-      {
-        sym(&d->get_initial_state, "spins_get_initial_state");
-        d->have_property = nullptr;
-        sym(&d->get_successors, "spins_get_successor_all");
-        sym(&d->get_state_size, "spins_get_state_size");
-        sym(&d->get_state_variable_name, "spins_get_state_variable_name");
-        sym(&d->get_state_variable_type, "spins_get_state_variable_type");
-        sym(&d->get_type_count, "spins_get_type_count");
-        sym(&d->get_type_name, "spins_get_type_name");
-        sym(&d->get_type_value_count, "spins_get_type_value_count");
-        sym(&d->get_type_value_name, "spins_get_type_value_name");
-      }
-    // dve2 and gal2C interfaces.
-    else
-      {
-        sym(&d->get_initial_state, "get_initial_state");
-        *reinterpret_cast<void**>(&d->have_property) =
-          lt_dlsym(h, "have_property");
-        sym(&d->get_successors, "get_successors");
-        sym(&d->get_state_size, "get_state_variable_count");
-        sym(&d->get_state_variable_name, "get_state_variable_name");
-        sym(&d->get_state_variable_type, "get_state_variable_type");
-        sym(&d->get_type_count, "get_state_variable_type_count");
-        sym(&d->get_type_name, "get_state_variable_type_name");
-        sym(&d->get_type_value_count, "get_state_variable_type_value_count");
-        sym(&d->get_type_value_name, "get_state_variable_type_value");
-      }
-
-    if (d->have_property && d->have_property())
-      throw std::runtime_error("Models with embedded properties "
-                               "are not supported.");
-
-    return { d };
+    return {std::make_shared<spins_interface>(file_arg)};
   }
 
+  ltsmin_kripkecube_ptr
+  ltsmin_model::kripkecube(std::vector<std::string> to_observe,
+                           const formula dead, int compress,
+                           unsigned int nb_threads) const
+  {
+    // Register the "dead" proposition.  There are three cases to
+    // consider:
+    //  * If DEAD is "false", it means we are not interested in finite
+    //    sequences of the system.
+    //  * If DEAD is "true", we want to check finite sequences as well
+    //    as infinite sequences, but do not need to distinguish them.
+    //  * If DEAD is any other string, this is the name a property
+    //    that should be true when looping on a dead state, and false
+    //    otherwise.
+    std::string dead_ap = "";
+    bool selfloopize = true;
+    if (dead == spot::formula::ff())
+      selfloopize = false;
+    else if (dead != spot::formula::tt())
+      dead_ap = dead.ap_name();
+
+    // Is dead proposition is already in to_observe?
+    bool add_dead = true;
+    for (auto it: to_observe)
+      if (it.compare(dead_ap))
+        add_dead = false;
+
+    if (dead_ap.compare("") != 0 && add_dead)
+      to_observe.push_back(dead_ap);
+
+    // Finally build the system.
+    return std::make_shared<spot::kripkecube<spot::cspins_state,
+                                             spot::cspins_iterator>>
+      (iface, compress, to_observe, selfloopize, dead_ap, nb_threads);
+  }
 
   kripke_ptr
   ltsmin_model::kripke(const atomic_prop_set* to_observe,
@@ -1122,7 +980,6 @@ namespace spot
   {
   }
 
-
   int ltsmin_model::state_size() const
   {
     return iface->get_state_size();
@@ -1157,5 +1014,4 @@ namespace spot
   {
     return iface->get_type_value_name(type, val);
   }
-
 }
