@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2014-2021 Laboratoire de Recherche et Développement
+// Copyright (C) 2014-2022 Laboratoire de Recherche et Développement
 // de l'Epita.
 //
 // This file is part of Spot, a model checking library.
@@ -21,13 +21,90 @@
 #include <spot/twa/twagraph.hh>
 #include <spot/tl/print.hh>
 #include <spot/misc/bddlt.hh>
+#include <spot/misc/timer.hh>
 #include <spot/twa/bddprint.hh>
+#include <spot/kripke/kripkegraph.hh>
 #include <spot/misc/escape.hh>
+#include <spot/priv/robin_hood.hh>
 #include <vector>
 #include <deque>
 
 using namespace std::string_literals;
 
+namespace
+{
+  using namespace spot;
+    // If LAST is false,
+    // it is guaranteed that there will be another src state
+    template<bool LAST>
+    void treat(std::vector<std::array<unsigned, 4>>& e_idx,
+                const twa_graph::graph_t::edge_vector_t& e_vec,
+                std::vector<unsigned>& e_chain,
+                std::vector<unsigned>& hash_of_state,
+                unsigned& idx,
+                unsigned s,
+                unsigned n_e)
+    {
+      assert(s < e_idx.size());
+      assert(idx < e_vec.size());
+      assert(e_chain.size() == e_vec.size());
+
+      //std::cout << s << "; " << idx << std::endl;
+
+      // Check if this state has outgoing transitions
+      if (s != e_vec[idx].src)
+        // Nothing to do
+        {
+          assert(!LAST);
+          return;
+        }
+
+      auto& s_idx = e_idx[s];
+      s_idx[0] = idx;
+
+      // helper
+      unsigned sub_idx[] = {-1u, -1u};
+
+      // All transitions of this state
+      while (true)
+        {
+          assert(idx < e_vec.size() + LAST);
+          if constexpr (!LAST)
+            {
+              if (e_vec[idx].src != s)
+                break;
+            }
+          else
+            {
+              if (idx == n_e)
+                break;
+            }
+
+          // Argh so many ifs
+          unsigned which = e_vec[idx].src == e_vec[idx].dst;
+          if (sub_idx[which] == -1u)
+            {
+              // First non-selflooping
+              sub_idx[which] = idx;
+              s_idx[1u+which] = idx;
+            }
+          else
+            {
+              // Continue the chained list
+              e_chain[sub_idx[which]] = idx;
+              sub_idx[which] = idx;
+            }
+          ++idx;
+        }
+      s_idx[3] = idx;
+
+      // Check if self-loops appeared.  We cannot hash
+      // states with self-loops.
+      if (s_idx[2] != -1u)
+        hash_of_state[s] = fnv<unsigned>::init;
+    }
+}
+
 namespace spot
 {
 
@@ -289,85 +366,367 @@ namespace spot
     g_.chain_edges_();
   }
 
-  unsigned twa_graph::merge_states()
+  unsigned twa_graph::merge_states(parallel_policy ppolicy)
   {
     if (!is_existential())
       throw std::runtime_error(
           "twa_graph::merge_states() does not work on alternating automata");
 
+#ifdef ENABLE_PTHREAD
+    const unsigned nthreads = ppolicy.nthreads();
+#else
+    (void) ppolicy;
+    constexpr unsigned nthreads = 1;
+#endif
+
     typedef graph_t::edge_storage_t tr_t;
-    g_.sort_edges_([](const tr_t& lhs, const tr_t& rhs)
+    g_.sort_edges_srcfirst_([](const tr_t& lhs, const tr_t& rhs)
     {
-      if (lhs.src < rhs.src)
-        return true;
-      if (lhs.src > rhs.src)
-        return false;
+      assert(lhs.src == rhs.src);
       if (lhs.acc < rhs.acc)
         return true;
       if (lhs.acc > rhs.acc)
         return false;
+      // compare with id?
       if (bdd_less_than_stable lt; lt(lhs.cond, rhs.cond))
         return true;
       if (rhs.cond != lhs.cond)
         return false;
-      // The destination must be sorted last
-      // for our self-loop optimization to work.
       return lhs.dst < rhs.dst;
-    });
+    }, nthreads);
     g_.chain_edges_();
 
+    const unsigned n_states = num_states();
+
+    // Edges are nicely chained and there are no erased edges
+    // -> We can work with the edge_vector
+
+    // Check if it is a game <-> "state-player" is defined.  If
+    // so, we can only merge states that belong to the same player.
+    // (We will use two hash maps in this case.)
+    auto sp = get_named_prop<std::vector<bool>>("state-player");
+
+    // The hashing is a bit delicat: We may only use the dst if it has
+    // no self-loop.  HASH_OF_STATE stores the hash associated to each
+    // state (by default its own number) or some common value if the
+    // state contains self-loop.
+    std::vector<unsigned> hash_of_state;
+    hash_of_state.reserve(n_states);
+    for (unsigned i = 0; i < n_states; ++i)
+      hash_of_state.push_back(i);
+
+    const auto& e_vec = edge_vector();
+    unsigned n_edges = e_vec.size();
+
+    // For each state we need 4 indices of the edge vector
+    // [first, first_non_sfirst_selflooplfloop, first_selfloop, end]
+    // The init value makes sure nothing is done for dead end states
+    std::vector<std::array<unsigned, 4>> e_idx(n_states, {-1u, -1u,
+                                                          -1u, -1u});
+    // Like a linked list holding the non-selfloop and selfloop transitions
+    std::vector<unsigned> e_chain(n_edges, -1u);
+
+    unsigned idx = 1;
+
+    // Edges are sorted with respect to src first
+    const unsigned n_high = e_vec.back().src;
+    for (auto s = 0u; s < n_high; ++s)
+      treat<false>(e_idx, e_vec, e_chain,
+                   hash_of_state, idx, s, n_edges);
+    // Last one
+    treat<true>(e_idx, e_vec, e_chain,
+                hash_of_state, idx, n_high, n_edges);
+
+    assert(idx == e_vec.size() && "Something went wrong during indexing");
+
+    unsigned n_player1 = 0u;
+    if (sp)
+      n_player1 = std::accumulate(sp->begin(), sp->end(), 0u);
+
+    // Represents which states share a hash
+    // Head is in the unordered_map,
+    // hash_linked_list is like a linked list structure
+    // of fake pointers
+
+    std::vector<unsigned> hash_linked_list(n_states, -1u);
+    typedef robin_hood::unordered_flat_map<size_t,
+                                           std::pair<unsigned,
+                                                     unsigned>> player_map;
+    // If the automaton is not a game, everything is assumed to be
+    // owned by player 0.
+    player_map map0;            // for player 0
+    player_map map1;            // for player 1
+
+    map0.reserve(n_states - n_player1);
+    map1.reserve(n_player1);
+
+    // Sadly we need to loop the edges twice since we have
+    // to check for self-loops before hashing
+
+    auto emplace = [&hash_linked_list](auto& m, auto h, auto s)
+      {
+        auto [it, inserted] = m.try_emplace(h, std::make_pair(s, s));
+        if (!inserted)
+          {
+            // We already have an entry with hash "h".   Link it
+            // to the new state.
+            unsigned idx = it->second.second; // tail of the list
+            assert(idx < s && "Must be monotone");
+            hash_linked_list[idx] = s;
+            it->second.second = s;
+          }
+      };
+
+    // Hash all states
+    constexpr unsigned shift = sizeof(size_t)/2 * CHAR_BIT;
+    for (auto s = 0u; s != n_states; ++s)
+      {
+        size_t h = fnv<size_t>::init;
+        const unsigned e = e_idx[s][3];
+        for (unsigned i = e_idx[s][0]; i != e; ++i)
+          {
+            // If size_t has 8byte and unsigned has 4byte
+            // then this works fine, otherwise there might be more collisions
+            size_t hh = hash_of_state[e_vec[i].dst];
+            hh <<= shift;
+            hh += e_vec[i].cond.id();
+            h ^= hh;
+            h *= fnv<size_t>::prime;
+            h ^= e_vec[i].acc.hash();
+            h *= fnv<size_t>::prime;
+          }
+        if (sp && (*sp)[s])
+          emplace(map1, h, s);
+        else
+          emplace(map0, h, s);
+      }
+    // All states that might possible be merged share the same hash
+    // Info hash coll
+    //std::cout << "Hash collission rate pre merge: "
+    //          << ((map0.size()+map1.size())/((float)n_states))
+    //          << '\n';
+
+
+    // Check whether we can merge two states
+    // and takes into account the self-loops
+    auto state_equal = [&e_vec, &e_chain, &e_idx](unsigned s1, unsigned s2,
+                                                  std::vector<char>& checked1,
+                                                  std::vector<char>& checked2)
+      {
+        auto edge_data_comp = [](const auto& lhs,
+                                 const auto& rhs)
+          {
+            if (lhs.acc < rhs.acc)
+              return true;
+            if (lhs.acc > rhs.acc)
+              return false;
+            // todo compare with id
+            if (bdd_less_than_stable lt; lt(lhs.cond, rhs.cond))
+              return true;
+            return false;
+          };
+
+        auto [i1, nsl1, sl1, e1] = e_idx[s1];
+        auto [i2, nsl2, sl2, e2] = e_idx[s2];
+
+        unsigned n_trans = e1 - i1;
+        if ((e2 - i2) != n_trans)
+          return false; // Different number of outgoing trans
+
+        // checked1/2 is one element larger than necessary;
+        // the last element (false) serves as a sentinel.
+        checked1.clear();
+        checked1.resize(n_trans + 1, false);
+        checked2.clear();
+        checked2.resize(n_trans + 1, false);
+
+        // Try to match self-loops
+        unsigned self_loops_matched = 0;
+        while ((sl1 < e1) && (sl2 < e2))
+          {
+            auto& data1 = e_vec[sl1].data();
+            auto& data2 = e_vec[sl2].data();
+            if (data1 == data2)
+              {
+                // Matched
+                ++self_loops_matched;
+                checked1[sl1 - i1] = true; //never touches last element
+                checked2[sl2 - i2] = true;
+                // Advance both
+                sl1 = e_chain[sl1];
+                sl2 = e_chain[sl2];
+              }
+            // Since edges are ordered on each side, aadvance
+            // the smallest side in case there is no match.
+            else if (edge_data_comp(data1, data2))
+              sl1 = e_chain[sl1];
+            else
+              sl2 = e_chain[sl2];
+          }
+
+        // If the matched self-loops cover all transitions, we can
+        // stop here.
+        if (self_loops_matched == n_trans)
+          return true;
+
+        // The remaining edges need to match exactly
+        unsigned idx1 = i1;
+        unsigned idx2 = i2;
+        while (((idx1 < e1) && (idx2 < e2)))
+          {
+            // More efficient version?
+            // Skip checked edges
+            // Last element serves as break
+            while (checked1[idx1 - i1])
+              ++idx1;
+            while (checked2[idx2 - i2])
+              ++idx2;
+            // If one is out of bounds, so is the other
+            if (idx1 == e1)
+              {
+                assert(idx2 == e2);
+                break;
+              }
+
+            if  ((e_vec[idx1].dst != e_vec[idx2].dst)
+                 || !(e_vec[idx1].data() == e_vec[idx2].data()))
+              return false;
+
+            // Advance
+            ++idx1;
+            ++idx2;
+          }
+        // All edges have bee paired
+        return true;
+      };
+
     const unsigned nb_states = num_states();
     std::vector<unsigned> remap(nb_states, -1U);
-    for (unsigned i = 0; i != nb_states; ++i)
-      {
-        auto out1 = out(i);
-        for (unsigned j = 0; j != i; ++j)
-          {
-            auto out2 = out(j);
-            if (std::equal(out1.begin(), out1.end(), out2.begin(), out2.end(),
-                           [](const edge_storage_t& a,
-                              const edge_storage_t& b)
-                           { return ((a.dst == b.dst
-                                      || (a.dst == a.src && b.dst == b.src))
-                                     && a.data() == b.data()); }))
-            {
-              remap[i] = (remap[j] != -1U) ? remap[j] : j;
 
-              // Because of the special self-loop tests we use above,
-              // it's possible that i can be mapped to remap[j] even
-              // if j was last compatible states found.  Consider the
-              // following cases, taken from an actual test case:
-              // 18 is equal to 5, 35 is equal to 18, but 35 is not
-              // equal to 5.
-              //
-              // State: 5
-              // [0&1&2] 8 {3}
-              // [!0&1&2] 10 {1}
-              // [!0&!1&!2] 18 {1}
-              // [!0&!1&2] 19 {1}
-              // [!0&1&!2] 20 {1}
-              //
-              // State: 18
-              // [0&1&2] 8 {3}
-              // [!0&1&2] 10 {1}
-              // [!0&!1&!2] 18 {1} // self-loop
-              // [!0&!1&2] 19 {1}
-              // [!0&1&!2] 20 {1}
-              //
-              // State: 35
-              // [0&1&2] 8 {3}
-              // [!0&1&2] 10 {1}
-              // [!0&!1&!2] 35 {1} // self-loop
-              // [!0&!1&2] 19 {1}
-              // [!0&1&!2] 20 {1}
-              break;
-            }
+    // Check all pair of states with compatible hash
+    auto check_ix = [&](unsigned ix, std::vector<unsigned>& v,
+                        std::vector<char>& checked1,
+                        std::vector<char>& checked2)
+      {
+        if (hash_linked_list[ix] == -1U) // no compatible state
+          return;
+
+        v.clear();
+        for (unsigned i = ix; i != -1U; i = hash_linked_list[i])
+          v.push_back(i);
+        const unsigned vs = v.size();
+
+        for (unsigned idx = 0; idx < vs; ++idx)
+          {
+            unsigned i = v[idx];
+            for (unsigned jdx = 0; jdx < idx; ++jdx)
+              {
+                unsigned j = v[jdx];
+                if (state_equal(j, i, checked1, checked2))
+                  {
+                    remap[i] = (remap[j] != -1U) ? remap[j] : j;
+
+                    // Because of the special self-loop tests we use
+                    // above, it's possible that i can be mapped to
+                    // remap[j] even if j was the last compatible
+                    // state found.  Consider the following cases,
+                    // taken from an actual test case: 18 is equal to
+                    // 5, 35 is equal to 18, but 35 is not equal to 5.
+                    //
+                    // State: 5
+                    // [0&1&2] 8 {3}
+                    // [!0&1&2] 10 {1}
+                    // [!0&!1&!2] 18 {1}
+                    // [!0&!1&2] 19 {1}
+                    // [!0&1&!2] 20 {1}
+                    //
+                    // State: 18
+                    // [0&1&2] 8 {3}
+                    // [!0&1&2] 10 {1}
+                    // [!0&!1&!2] 18 {1} // self-loop
+                    // [!0&!1&2] 19 {1}
+                    // [!0&1&!2] 20 {1}
+                    //
+                    // State: 35
+                    // [0&1&2] 8 {3}
+                    // [!0&1&2] 10 {1}
+                    // [!0&!1&!2] 35 {1} // self-loop
+                    // [!0&!1&2] 19 {1}
+                    // [!0&1&!2] 20 {1}
+                    break;
+                  }
+              }
           }
-      }
+      };
+
+    auto upd = [](auto& b, const auto&e, unsigned it)
+      {
+        while ((it > 0) & (b != e))
+          {
+            --it;
+            ++b;
+          }
+      };
+
+    auto worker = [&upd, check_ix, nthreads](unsigned pid,
+                                             auto beg1, auto end1,
+                                             auto beg0, auto end0)
+    {
+      // Temporary storage for list of edges to reduce cache misses
+      std::vector<unsigned> v;
+      // Vector reused by all invocations of state_equal to mark edges
+      // that have been matched already.
+      std::vector<char> checked1;
+      std::vector<char> checked2;
+      upd(beg1, end1, pid);
+      upd(beg0, end0, pid);
+      for (; beg1 != end1; upd(beg1, end1, nthreads))
+        check_ix(beg1->second.first, v, checked1, checked2);
+      for (; beg0 != end0; upd(beg0, end0, nthreads))
+        check_ix(beg0->second.first, v, checked1, checked2);
+      };
+
+    {
+      auto beg1 = map1.begin();
+      auto end1 = map1.end();
+      auto beg0 = map0.begin();
+      auto end0 = map0.end();
+
+#ifndef ENABLE_PTHREAD
+      (void) nthreads;
+#else
+      if (nthreads <= 1)
+        {
+#endif // ENABLE_PTHREAD
+          worker(0, beg1, end1, beg0, end0);
+#ifdef ENABLE_PTHREAD
+        }
+      else
+        {
+          static auto tv = std::vector<std::thread>();
+          assert(tv.empty());
+          tv.resize(nthreads);
+          for (unsigned pid = 0; pid < nthreads; ++pid)
+            tv[pid] = std::thread(
+              [worker, pid, beg1, end1, beg0, end0]()
+              {
+                worker(pid, beg1, end1, beg0, end0);
+                return;
+              });
+          for (auto& t : tv)
+            t.join();
+          tv.clear();
+        }
+#endif // ENABLE_PTHREAD
+    }
 
     for (auto& e: edges())
       if (remap[e.dst] != -1U)
-        e.dst = remap[e.dst];
+        {
+          assert((!sp || (sp->at(e.dst) == sp->at(remap[e.dst])))
+                 && "States do not have the same owner");
+          e.dst = remap[e.dst];
+        }
 
     if (remap[get_init_state_number()] != -1U)
       set_init_state(remap[get_init_state_number()]);
@@ -382,6 +741,10 @@ namespace spot
     unsigned merged = num_states() - st;
     if (merged)
       defrag_states(remap, st);
+    // Info hash coll 2
+    //std::cout << "Hash collission rate post merge: "
+    //          << ((map0.size()+map1.size())/((float)num_states()))
+    //          << '\n';
     return merged;
   }
 
@@ -465,7 +828,7 @@ namespace spot
         comp_classes_.clear();
         // get all compatible classes
         // Candidate classes share a hash
-        // A state is compatible to a class if it is compatble
+        // A state is compatible to a class if it is compatible
         // to any of its states
         auto& cand_classes = equiv_class_[hi];
         unsigned n_c_classes = cand_classes.size();
@@ -713,8 +1076,16 @@ namespace spot
             bool useless = true;
             while (t)
               {
-                // An edge is useful if all its
-                // destinations are useful.
+                // Erase any false edge, except self-loops (which can
+                // be used to store colors on state without successor
+                // with state-based acceptance).
+                if (t->cond == bddfalse && t->src != t->dst)
+                  {
+                    t.erase();
+                    continue;
+                  }
+                // A non-false edge is useful if all its destinations
+                // are useful.
                 bool usefuledge = true;
                 for (unsigned d: univ_dests(t->dst))
                   if (!useful[d])
@@ -908,32 +1279,22 @@ namespace spot
           }
         std::swap(*hs, hs2);
       }
-    if (auto os = get_named_prop<std::vector<unsigned>>("original-states"))
-      {
-        unsigned size = os->size();
-        for (unsigned s = 0; s < size; ++s)
-          {
-            unsigned dst = newst[s];
-            if (dst == s || dst == -1U)
-              continue;
-            assert(dst < s);
-            (*os)[dst] = (*os)[s];
-          }
-        os->resize(used_states);
-      }
-    if (auto dl = get_named_prop<std::vector<unsigned>>("degen-levels"))
-      {
-        unsigned size = dl->size();
-        for (unsigned s = 0; s < size; ++s)
-          {
-            unsigned dst = newst[s];
-            if (dst == s || dst == -1U)
-              continue;
-            assert(dst < s);
-            (*dl)[dst] = (*dl)[s];
-          }
-        dl->resize(used_states);
-      }
+    for (const char* prop: {"original-classes",
+                            "original-states",
+                            "degen-levels"})
+      if (auto os = get_named_prop<std::vector<unsigned>>(prop))
+        {
+          unsigned size = os->size();
+          for (unsigned s = 0; s < size; ++s)
+            {
+              unsigned dst = newst[s];
+              if (dst == s || dst == -1U)
+                continue;
+              assert(dst < s);
+              (*os)[dst] = (*os)[s];
+            }
+          os->resize(used_states);
+        }
     if (auto ss = get_named_prop<std::vector<unsigned>>("simulated-states"))
       {
         for (auto& s : *ss)
@@ -944,6 +1305,22 @@ namespace spot
               s = newst[s];
           }
       }
+    // Reassign the state-players
+    if (auto sp = get_named_prop<std::vector<bool>>("state-player"))
+      {
+        const auto ns = (unsigned) used_states;
+        const auto sps = (unsigned) sp->size();
+        assert(ns <= sps);
+        assert(sps == newst.size());
+
+        for (unsigned i = 0; i < sps; ++i)
+          {
+            if (newst[i] == -1u)
+              continue;
+            (*sp)[newst[i]] = (*sp)[i];
+          }
+        sp->resize(ns);
+      }
     init_number_ = newst[init_number_];
     g_.defrag_states(newst, used_states);
   }
@@ -1275,6 +1652,17 @@ namespace spot
           return p.first->second;
         };
 
+      // If the input is a kripke_graph and the number of states is
+      // not restricted, predeclare all states to keep their
+      // numbering, and also copy unreachable states.
+      if (max_states == -1U)
+        if (auto kg = std::dynamic_pointer_cast<const kripke_graph>(aut))
+          {
+            unsigned ns = kg->num_states();
+            for (unsigned s = 0; s < ns; ++s)
+              new_state(kg->state_from_number(s));
+          }
+
       out->set_init_state(new_state(aut->get_init_state()));
       while (!todo.empty())
         {
@@ -1313,7 +1701,6 @@ namespace spot
             }
         }
 
-
       auto s = seen.begin();
       while (s != seen.end())
         {
