@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2017-2021 Laboratoire de Recherche et Développement
+// Copyright (C) 2017-2022 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -30,6 +30,7 @@
 #include <fstream>
 #include <string>
 #include <sstream>
+#include <initializer_list>
 
 #include <spot/twa/twagraph.hh>
 #include <spot/misc/bddlt.hh>
@@ -41,6 +42,13 @@
 #define STR_(x) STR(x)
 #define STR_LINE STR_(__LINE__)
 
+//#define TRACE
+#ifdef TRACE
+#  define trace std::cerr
+#else
+#  define trace while (0) std::cerr
+#endif
+
 namespace
 {
   using namespace spot;
@@ -459,6 +467,7 @@ namespace spot
   aig::roll_back_(safe_point sf, bool do_stash)
   {
     // todo specialise for safe_all?
+    trace << "Roll back to sf: " << sf.first << "; " << sf.second << '\n';
     safe_stash ss;
     auto& [gates, vardict, negs] = ss;
     if (do_stash)
@@ -480,6 +489,7 @@ namespace spot
         // Copy the gates
         std::copy(and_gates_.begin()+sf.second, and_gates_.end(),
                   gates.begin());
+        trace << "Safed " << gates.size() << '\n';
       }
     // 1. Delete all literals
     // max_var_old was used before
@@ -489,6 +499,8 @@ namespace spot
     // 2. Set back the gates
     and_gates_.erase(and_gates_.begin() + sf.second, and_gates_.end());
     max_var_ = sf.first;
+    trace << "After rollback: \n" << and_gates_.size() << " gates and\n"
+          << max_var_ << " variables\n\n";
     return ss;
   }
 
@@ -497,6 +509,8 @@ namespace spot
   {
     // Do some check_ups
     auto& [gates, vardict, _] = ss;
+    trace << "Reapplying sf: " << sf.first << "; " << sf.second
+          << "\nwith " << gates.size() << " additional gates.\n\n";
     assert(gates.size() == vardict.size());
     assert(sf.first == max_var_);
     assert(sf.second == and_gates_.size());
@@ -511,6 +525,7 @@ namespace spot
     and_gates_.insert(and_gates_.end(),
                       gates.begin(), gates.end());
     max_var_ = new_max_var_;
+    trace << "New Ngates: " << num_gates() << '\n';
   }
 
   void aig::set_output(unsigned i, unsigned v)
@@ -622,19 +637,44 @@ namespace spot
     // De-morgan
     // !(!v&low | v&high) = !(!v&low) & !(v&high)
     // !v&low | v&high = !(!(!v&low) & !(v&high))
+    // note that if low or high are T
+    // we can simplify the formula
+    // given that low / high is T
+    // then !(!v&low) & !(v&high) can be simplified to
+    // !v&low | v&high = !v | high / low | v
+    // = !(v & !high) /  !(!low & !v)
+    // The case when low / high is ⊥ is automatically treated
     auto b_it = bdd2var_.find(b.id());
     if (b_it != bdd2var_.end())
       return b_it->second;
 
-    // todo
-//    unsigned v_var = bdd2var_.at(bdd_var(b));
     unsigned v_var = bdd2var_.at(bdd_ithvar(bdd_var(b)).id());
 
     unsigned b_branch_var[2] = {bdd2INFvar(bdd_low(b)),
                                 bdd2INFvar(bdd_high(b))};
 
-    unsigned r = aig_not(aig_and(v_var, b_branch_var[1]));
-    unsigned l = aig_not(aig_and(aig_not(v_var), b_branch_var[0]));
+    unsigned l;
+    unsigned r;
+
+    if (b_branch_var[0] == aig_true())
+      {
+        // low == T
+        l = v_var;
+        r = aig_not(b_branch_var[1]);
+      }
+    else if (b_branch_var[1] == aig_true())
+      {
+        // high == T
+        l = aig_not(b_branch_var[0]);
+        r = aig_not(v_var);
+      }
+    else
+      {
+        // General case
+        r = aig_not(aig_and(v_var, b_branch_var[1]));
+        l = aig_not(aig_and(aig_not(v_var), b_branch_var[0]));
+      }
+
     return aig_not(aig_and(l, r));
   }
 
@@ -698,7 +738,6 @@ namespace spot
     while ((prod = cond.next()) != bddfalse)
       plus_vars_.push_back(cube2var_(prod,
                                      use_split_off == 2 ? 0 : use_split_off));
-
     // Done building -> sum them
     return aig_or(plus_vars_);
   }
@@ -709,11 +748,20 @@ namespace spot
   {
     // Before doing anything else, let us check if one the variables
     // already exists in which case we are done
+#ifdef TRACE
+    trace << "encoding one of \n";
+    for (const auto& c: c_alt)
+      trace << c << '\n';
+#endif
+
     for (const bdd& cond : c_alt)
       {
         auto it = bdd2var_.find(cond.id());
         if (it != bdd2var_.end())
-          return it->second;
+          {
+            trace << "Condition already encoded -> Direct return\n\n";
+            return it->second;
+          }
       }
 
     safe_point sf = get_safe_point_();
@@ -732,9 +780,6 @@ namespace spot
            && "Cannot convert the given method. "
               "Only 0,1 and 2 are currently supported");
 
-    const auto negate = use_dual ? std::vector<bool>{false}
-                                 : std::vector<bool>{false, true};
-
     auto enc_1 = [&](const bdd& b,
                      const char m)
       {
@@ -751,41 +796,60 @@ namespace spot
 
     std::vector<bdd> cond_parts;
     std::vector<unsigned> cond_vars;
-    for (bool do_negate : negate)
-      for (const bdd& b : c_alt)
-        {
-          bdd b_used = do_negate ? bdd_not(b) : b;
-          cond_parts.clear();
-          split_cond_(b_used,
-                      use_split_off != 1 ?  use_split_off : 0, cond_parts);
+    //for (bool do_negate : (use_dual ? std::initializer_list<bool>{false, true}
+    //                                : std::initializer_list<bool>{false}))
+    for (unsigned neg_counter = 0; neg_counter <= 0 + use_dual; ++neg_counter)
+      {
+        bool do_negate = neg_counter;
+        for (const bdd& b : c_alt)
+          {
+            bdd b_used = do_negate ? bdd_not(b) : b;
+            cond_parts.clear();
+            split_cond_(b_used,
+                        use_split_off != 1 ?  use_split_off : 0, cond_parts);
 
-          for (auto m : used_m)
-            {
-              cond_vars.clear();
-              for (const bdd& cpart : cond_parts)
-                {
-                  cond_vars.push_back(enc_1(cpart, m));
-                  if (num_gates() >= ngates_min)
-                    break; // Cannot be optimal
-                }
-              // Compute the and if there is still hope
-              unsigned this_res = -1u;
-              if (num_gates() < ngates_min)
-                this_res = aig_and(cond_vars);
-
-              if (num_gates() < ngates_min)
-                {
-                  // This is the new best
-                  res_var = do_negate ? aig_not(this_res) : this_res;
-                  ngates_min = num_gates();
-                  ss_min = roll_back_(sf, true);
-                }
-              else
-                // Reset the computations
-                roll_back_(sf, false);
-            } // Encoding styles
-        } // alternatives
-    // end do_negate
+            for (auto m : used_m)
+              {
+                cond_vars.clear();
+                for (const bdd& cpart : cond_parts)
+                  {
+                    cond_vars.push_back(enc_1(cpart, m));
+                    if (num_gates() >= ngates_min)
+                      break; // Cannot be optimal
+                  }
+                // Compute the and if there is still hope
+                auto this_res = -1u;
+                if (num_gates() < ngates_min)
+                  this_res = aig_and(cond_vars);
+                // Check if after adding these gates
+                // the circuit is still smaller
+                if (num_gates() < ngates_min)
+                  {
+                    // This is the new best
+                    assert(this_res != -1u);
+                    res_var = do_negate ? aig_not(this_res) : this_res;
+                    ngates_min = num_gates();
+                    trace << "Found new best encoding with\nneg: "
+                          << do_negate << "\nmethod: " << (m == 0 ? "INF"
+                                                                  : "ISOP")
+                          << "\nalt: " << b
+                          << "\nNgates: " << num_gates() << "\n\n";
+                    ss_min = roll_back_(sf, true);
+                  }
+                else
+                  // Reset the computations
+                  {
+                    trace << "Method \nneg: "
+                          << do_negate << "\nmethod: " << (m == 0 ? "INF"
+                                                                  : "ISOP")
+                          << "\nalt: " << b
+                          << "\nNgates: " << num_gates()
+                          << " discarded.\n\n";
+                    roll_back_(sf, false);
+                  }
+              } // Encoding styles
+          } // alternatives
+      } // end do_negate
 
     // Reapply the best result
     reapply_(sf, ss_min);
@@ -1359,7 +1423,7 @@ namespace spot
       {
         unsigned var_g = gate_var(i);
         state_[var_g] = state_[and_gates_[i].first]
-                        & state_[and_gates_[i].second];
+                        && state_[and_gates_[i].second];
         state_[aig_not(var_g)] = !state_[var_g];
       }
     // Update latches
@@ -1379,7 +1443,8 @@ namespace
   using namespace spot;
 
   // We have to decide which actual output to use:
-  // Heuristic : Use the assignment having the least highs
+  // Heuristic : Use the assignment having the smallest cost
+  // according to satoneshortest
   static std::vector<std::vector<bdd>>
   choose_outc(const std::vector<std::pair<const_twa_graph_ptr, bdd>>&
                   strat_vec,
@@ -1387,18 +1452,44 @@ namespace
   {
     std::vector<std::vector<bdd>> used_outc;
 
-    for (auto&& astrat : strat_vec)
+    for (const auto& astrat : strat_vec)
       {
         used_outc.emplace_back(astrat.first->num_edges()+1);
         auto& this_outc = used_outc.back();
 
-        for (auto&& e: astrat.first->edges())
+        // Check if split or separated
+        if (auto sp_ptr =
+              astrat.first->get_named_prop<region_t>("state-player"))
           {
-            assert(e.cond != bddfalse);
-            bdd bout = bdd_exist(e.cond, all_inputs);
-            // low is good, dc is ok, high is bad
-            this_outc[astrat.first->edge_number(e)] =
-                bdd_satoneshortest(bout, 0, 1, 5);
+            // Split -> We only need outs for env edges
+            auto sp = *sp_ptr;
+            const unsigned N = astrat.first->num_states();
+            for (unsigned s = 0; s < N; ++s)
+              {
+                if (sp[s])
+                  continue; //Player state -> nothing to do
+                for (const auto& e : astrat.first->out(s))
+                  {
+                    assert(e.cond != bddfalse);
+                    bdd bout = astrat.first->out(e.dst).begin()->cond;
+                    assert(bout != bddfalse);
+                    // low is good, dc is ok, high is bad
+                    this_outc[astrat.first->edge_number(e)] =
+                        bdd_satoneshortest(bout, 0, 1, 5);
+                  }
+              }
+          }
+        else
+          {
+            // separated
+            for (const auto& e: astrat.first->edges())
+              {
+                assert(e.cond != bddfalse);
+                bdd bout = bdd_exist(e.cond, all_inputs);
+                // low is good, dc is ok, high is bad
+                this_outc[astrat.first->edge_number(e)] =
+                    bdd_satoneshortest(bout, 0, 1, 5);
+              }
           }
       }
     //Done
@@ -1409,6 +1500,7 @@ namespace
   state_to_vec(std::vector<unsigned>& v, unsigned s,
                unsigned offset)
   {
+    assert(s != -1u && "-1u is not a valid sstate");
     v.clear();
     unsigned i = offset;
     while (s > 0)
@@ -1466,8 +1558,10 @@ namespace
         }
     // This checks the acc again, but does more and is to
     // costly for non-debug mode
+    // We can also handle split machines
     assert(std::all_of(strat_vec.begin(), strat_vec.end(),
-           [](const auto& p){ return is_separated_mealy(p.first); }));
+           [](const auto& p){ return is_separated_mealy(p.first)
+                                     || is_split_mealy(p.first); }));
 
     // get the propositions
     std::vector<std::string> input_names;
@@ -1558,30 +1652,48 @@ namespace
     unsigned n_latches = 0;
     for (const auto& astrat : strat_vec)
       {
-        state_numbers.emplace_back();
-        state_numbers.back().reserve(astrat.first->num_states());
+        const unsigned N = astrat.first->num_states();
+        state_numbers.emplace_back(N, -1u);
+        auto& sn = state_numbers.back();
         unsigned max_index = 0;
         // Check if named
+        auto sp_ptr = astrat.first->get_named_prop<region_t>("state-player");
         if (const auto* s_names =
                 astrat.first->
                   get_named_prop<std::vector<std::string>>("state-names"))
           {
-            std::transform(s_names->cbegin(), s_names->cend(),
-                           std::back_inserter(state_numbers.back()),
-                           [&max_index](const auto& sn)
-                           {
-                             unsigned su = std::stoul(sn);
-                             max_index = std::max(max_index, su);
-                             return su;
-                           });
+            for (unsigned n = 0; n < N; ++n)
+              {
+                if (sp_ptr && (*sp_ptr)[n])
+                  continue;
+                // Remains -1u
+                unsigned su = std::stoul((*s_names)[n]);
+                max_index = std::max(max_index, su);
+                sn[n] = su;
+              }
             ++max_index;
           }
         else
           {
-            max_index = astrat.first->num_states();
-            state_numbers.back().resize(astrat.first->num_states());
-            std::iota(state_numbers.back().begin(),
-                      state_numbers.back().end(), 0);
+            if (sp_ptr)
+              {
+                auto sp = *sp_ptr;
+                // Split
+                unsigned n_next = 0;
+                // Player -1u
+                // Env: Succesively numbered according to appearance
+                for (unsigned n = 0; n < N; ++n)
+                  if (!sp[n])
+                    sn[n] = n_next++;
+                max_index = n_next;
+              }
+            else
+              {
+                std::iota(state_numbers.back().begin(),
+                          state_numbers.back().end(), 0);
+                max_index = N;
+              }
+
             // Ensure 0 <-> init state
             std::swap(state_numbers.back()[0],
                       state_numbers.back()[astrat.first->
@@ -1621,6 +1733,8 @@ namespace
         auto&& [astrat, aouts] = strat_vec[i];
         const auto& sn = state_numbers.at(i);
 
+        auto sp_ptr = astrat->get_named_prop<region_t>("state-player");
+
         auto latchoff = latch_offset[i];
 #ifndef NDEBUG
         auto latchoff_next = latch_offset.at(i+1);
@@ -1628,6 +1742,8 @@ namespace
         auto alog2n = log2n[i];
         auto state2bdd = [&](auto s)
           {
+            assert((!sp_ptr || !(*sp_ptr)[s])
+                    && "Only env states need to be encoded.");
             auto s2 = sn[s];
             bdd b = bddtrue;
             for (unsigned j = 0; j < alog2n; ++j)
@@ -1654,13 +1770,19 @@ namespace
 
         for (unsigned s = 0; s < astrat->num_states(); ++s)
           {
+            // Player state -> continue
+            if (sp_ptr && (*sp_ptr)[s])
+              continue;
             // Convert src state to bdd
             bdd src_bdd = state2bdd(s);
 
             for (const auto& e : astrat->out(s))
               {
                 // High latches of dst
-                state_to_vec(next_state_vec, sn[e.dst], latchoff);
+                state_to_vec(next_state_vec,
+                             sp_ptr ? sn[astrat->out(e.dst).begin()->dst]
+                                    : sn[e.dst],
+                             latchoff);
 
                 // Get high outs depending on cond
                 output_to_vec(out_vec, out_dc_vec,
@@ -1695,6 +1817,7 @@ namespace
       bool use_dual = false;
       bool use_dontcare = false;
       int use_split_off = 0;
+      std::string s;
     };
 
     auto to_treat = [&mode]()
@@ -1708,6 +1831,8 @@ namespace
         while (std::getline(s, buffer, ','))
           {
             tr_opt this_opt;
+            // Store raw info
+            this_opt.s = buffer;
             std::stringstream s2;
             s2 << buffer;
             std::getline(s2, buffer2, '+');
@@ -1807,15 +1932,16 @@ namespace
               };
 
             // Create the vars
-            std::vector<bdd> alt_conds(amodedescr.use_dontcare ? 1 : 2);
             for (unsigned i = 0; i < n_outs; ++i)
               {
+                trace << "Assign out " << i << '\n';
                 if (circuit.num_gates() > min_gates)
                   break;
                 circuit.set_output(i, bdd2var(out[i], out_dc[i]));
               }
             for (unsigned i = 0; i < n_latches; ++i)
               {
+                trace << "Assign latch " << i << '\n';
                 if (circuit.num_gates() > min_gates)
                   break;
                 circuit.set_next_latch(i, bdd2var(latch[i], bddfalse));
@@ -1825,6 +1951,8 @@ namespace
         // Overwrite the stash if we generated less gates
         if (circuit.num_gates() < min_gates)
           {
+            trace << "New best mode: " << amodedescr.s
+                  << " with Ngates: " << circuit.num_gates() << '\n';
             min_gates = circuit.num_gates();
             ss = circuit.roll_back_(sf, true);
             bdd2var_min = bdd2var;
@@ -1834,6 +1962,8 @@ namespace
       }
     //Use the best sol
     circuit.reapply_(sf, ss);
+    trace << "Finished encoding, reasssigning\n"
+          << "Final gate count is " << circuit.num_gates() << '\n';
     // Reset them
     for (unsigned i = 0; i < n_outs; ++i)
       circuit.set_output(i, bdd2var_min(out[i], out_dc[i]));
