@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2015-2021 Laboratoire de Recherche et
+// Copyright (C) 2015-2022 Laboratoire de Recherche et
 // Développement de l'Epita.
 //
 // This file is part of Spot, a model checking library.
@@ -476,7 +476,7 @@ namespace spot
           std::vector<int> tmp;
           while (brace >= 0)
             {
-              // FIXME is not there a smarter way?
+              // FIXME: is there a smarter way?
               tmp.insert(tmp.begin(), brace);
               brace = s.braces_[brace];
             }
@@ -548,6 +548,35 @@ namespace spot
       return res;
     }
 
+    std::vector<unsigned>*
+    classify(const power_set& states,
+             const std::vector<unsigned>* original_class)
+    {
+      unsigned sz = states.size();
+      auto classes = new std::vector<unsigned>(sz);
+      std::vector<std::set<unsigned>> state_sets(sz);
+      for (const auto& p: states)
+        {
+          // Get the set of original stats of this
+          std::set<unsigned> s;
+          for (auto& n: p.first.nodes_)
+            {
+              unsigned st = n.first;
+              if (original_class)
+                st = (*original_class)[st];
+              s.insert(st);
+            }
+          state_sets[p.second] = std::move(s);
+        }
+      std::map<std::set<unsigned>, unsigned> seensets;
+      for (unsigned s = 0; s < sz; ++s)
+        {
+          auto p = seensets.emplace(state_sets[s], s);
+          (*classes)[s] = p.first->second;
+        }
+      return classes;
+    }
+
     class safra_support
     {
       const std::vector<bdd>& state_supports;
@@ -752,7 +781,7 @@ namespace spot
   bool
   safra_state::operator<(const safra_state& other) const
   {
-    // FIXME what is the right, if any, comparison to perform?
+    // FIXME: what is the right, if any, comparison to perform?
     return braces_ == other.braces_ ? nodes_ < other.nodes_
                                     : braces_ < other.braces_;
   }
@@ -812,7 +841,8 @@ namespace spot
                    bool pretty_print, bool use_scc,
                    bool use_simulation, bool use_stutter,
                    const output_aborter* aborter,
-                   int trans_pruning)
+                   int trans_pruning,
+                   bool want_classes)
   {
     if (!a->is_existential())
       throw std::runtime_error
@@ -857,7 +887,7 @@ namespace spot
           // NB spot::simulation() does not remove unreachable states, as it
           // would invalidate the contents of 'implications'.
           // so we need to explicitly test for unreachable states
-          // FIXME based on the scc_info, we could remove the unreachable
+          // FIXME: based on the scc_info, we could remove the unreachable
           // states, both in the input automaton and in 'implications'
           // to reduce the size of 'implies'.
           if (!scc.reachable_state(i))
@@ -892,7 +922,7 @@ namespace spot
     std::vector<bdd> support(aut->num_states());
     if (use_stutter && aut->prop_stutter_invariant())
       {
-        // FIXME this could be improved
+        // FIXME: this could be improved
         // supports of states should account for possible stuttering if we plan
         // to use stuttering invariance
         for (unsigned c = 0; c != scc.scc_count(); ++c)
@@ -996,6 +1026,27 @@ namespace spot
 
     if (pretty_print)
       res->set_named_prop("state-names", print_debug(aut, seen));
+
+    if (want_classes)
+      {
+        auto* ptr =
+          aut->get_named_prop<std::vector<unsigned>>("original-states");
+        if (ptr && ptr->size() != aut->num_states())
+          throw std::runtime_error("tgba_determinize(): "
+                                   "input's \"original-states\" property "
+                                   "has unexpected size");
+        if (!ptr)
+          {
+            ptr =
+              aut->get_named_prop<std::vector<unsigned>>("original-classes");
+            if (ptr && ptr->size() != aut->num_states())
+              throw std::runtime_error("tgba_determinize(): "
+                                       "input's \"original-classes\" property "
+                                       "has unexpected size");
+          }
+        res->set_named_prop("original-classes", classify(seen, ptr));
+      }
+
     return res;
   }
 }
