@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2013-2021 Laboratoire de Recherche
+// Copyright (C) 2013-2023 Laboratoire de Recherche
 // et Développement de l'Epita.
 //
 // This file is part of Spot, a model checking library.
@@ -98,11 +98,7 @@ namespace spot
           return true;
         if (this->acc_ref > other.acc_ref)
           return false;
-        if (this->acc_cand < other.acc_cand)
-          return true;
-        if (this->acc_cand > other.acc_cand)
-          return false;
-        return false;
+        return this->acc_cand < other.acc_cand;
       }
 
     };
@@ -864,7 +860,7 @@ namespace spot
 #if TRACE
       std::fstream out("dtwa-sat.dbg",
                        std::ios_base::trunc | std::ios_base::out);
-      out.exceptions(std::ifstream::failbit | std::ifstream::badbit);
+      out.exceptions(std::ofstream::failbit | std::ofstream::badbit);
 #endif
       std::map<int, acc_cond::mark_t> state_acc;
       std::set<src_cond> seen_trans;
