@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2017-2021 Laboratoire de Recherche et Developpement
+// Copyright (C) 2017-2022 Laboratoire de Recherche et Developpement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -25,7 +25,7 @@ namespace spot
 {
   namespace
   {
-    enum genem_version_t { spot28, atva19, spot29, spot210 };
+    enum genem_version_t { spot28, atva19, spot29, spot210, spot211 };
     static genem_version_t genem_version = spot29;
   }
 
@@ -33,6 +33,8 @@ namespace spot
   {
     if (emversion == nullptr || !strcasecmp(emversion, "spot29"))
       genem_version = spot29;
+    else if (!strcasecmp(emversion, "spot211"))
+      genem_version = spot211;
     else if (!strcasecmp(emversion, "spot210"))
       genem_version = spot210;
     else if (!strcasecmp(emversion, "spot28"))
@@ -41,7 +43,8 @@ namespace spot
       genem_version = atva19;
     else
       throw std::invalid_argument("generic_emptiness_check version should be "
-                                  "one of {spot28, atva19, spot29, spot210}");
+                                  "one of {spot28, atva19, spot29, spot210, "
+                                  "spot211}");
   }
 
   namespace
@@ -84,6 +87,8 @@ namespace spot
     scc_split_check(const scc_info& si, unsigned scc, const acc_cond& acc,
                     Extra extra, acc_cond::mark_t tocut)
     {
+      if (genem_version == spot211 || genem_version == spot210)
+        tocut |= acc.fin_unit();
       scc_and_mark_filter filt(si, scc, tocut);
       filt.override_acceptance(acc);
       scc_info upper_si(filt, EarlyStop
@@ -118,13 +123,27 @@ namespace spot
             // Try to accept when Fin(fo) == true
             acc_cond::mark_t fo_m = {(unsigned) fo};
             if (!scc_split_check<EarlyStop, Extra>
-                (si, scc, fpart.remove(fo_m, true), extra, fo_m))
+                (si, scc, fpart, extra, fo_m))
               if constexpr (EarlyStop)
                 return false;
             // Try to accept when Fin(fo) == false
             acc = acc.force_inf(fo_m);
           }
         while (!acc.is_f());
+      else if (genem_version == spot211)
+        {
+          do
+            {
+              auto [fo, fpart, rest] = acc.fin_unit_one_split();
+              acc_cond::mark_t fo_m = {(unsigned) fo};
+              if (!scc_split_check<EarlyStop, Extra>
+                  (si, scc, fpart, extra, fo_m))
+                if constexpr (EarlyStop)
+                  return false;
+              acc = rest;
+            }
+          while (!acc.is_f());
+        }
       else if (genem_version == spot29)
         do
           {
@@ -290,5 +309,82 @@ namespace spot
     return !scc_split_check_filtered<false>(upper_si, forced_acc, callback, {});
   }
 
+  // return ⊤ if there exists at least one accepting transition.
+  static bool
+  accepting_transitions_aux(const scc_info &si, unsigned scc,
+                                const acc_cond acc,
+                                acc_cond::mark_t removed_colors,
+                                acc_cond::mark_t tocut,
+                                std::vector<bool> &accepting_transitions,
+                                const bitvect& kept)
+  {
+    bool result = false;
+    scc_and_mark_filter filt(si, scc, tocut, kept);
+    filt.override_acceptance(acc);
+    scc_info upper_si(filt, scc_info_options::ALL);
+    for (unsigned sc = 0; sc < upper_si.scc_count(); ++sc)
+      result |= accepting_transitions_scc(upper_si, sc, acc, removed_colors,
+                                accepting_transitions, kept);
+    return result;
+  }
 
+  bool
+  accepting_transitions_scc(const scc_info &si, unsigned scc,
+                              const acc_cond aut_acc,
+                              acc_cond::mark_t removed_colors,
+                              std::vector<bool>& accepting_transitions,
+                              const bitvect& kept)
+  {
+    // The idea is the same as in is_scc_empty()
+    bool result = false;
+    acc_cond::mark_t sets = si.acc_sets_of(scc);
+    acc_cond acc = aut_acc.restrict_to(sets);
+    acc = acc.remove(si.common_sets_of(scc), false);
+
+    auto inner_edges = si.inner_edges_of(scc);
+
+    if (si.is_trivial(scc))
+      return false;
+    if (acc.is_t() || acc.accepting(acc.get_acceptance().used_sets()))
+    {
+      for (auto& e : inner_edges)
+        if ((e.acc & removed_colors) == acc_cond::mark_t {})
+          accepting_transitions[si.get_aut()->edge_number(e)] = true;
+      return true;
+    }
+    else if (acc.is_f())
+      return false;
+    acc_cond::acc_code rest = acc_cond::acc_code::f();
+    for (const acc_cond& disjunct: acc.top_disjuncts())
+      if (acc_cond::mark_t fu = disjunct.fin_unit())
+        result |= accepting_transitions_aux(si, scc, acc.remove(fu, true),
+                                  (removed_colors | fu), fu,
+                                  accepting_transitions, kept);
+      else
+        rest |= disjunct.get_acceptance();
+    if (!rest.is_f())
+    {
+      acc_cond::mark_t m = { (unsigned) acc.fin_one() };
+      result |= accepting_transitions_aux(si, scc, acc.remove(m, true),
+                                (removed_colors | m), m, accepting_transitions,
+                                kept);
+      result |= accepting_transitions_scc(si, scc, acc.remove(m, false),
+                                removed_colors, accepting_transitions,
+                                kept);
+    }
+    return result;
+  }
+
+  std::vector<bool>
+  accepting_transitions(const const_twa_graph_ptr aut, acc_cond cond)
+  {
+    auto aut_vector_size = aut->edge_vector().size();
+    std::vector<bool> result(aut_vector_size, false);
+    auto kept = make_bitvect(aut_vector_size);
+    scc_info si(aut);
+    for (unsigned scc = 0; scc < si.scc_count(); ++scc)
+      accepting_transitions_scc(si, scc, cond, {}, result, *kept);
+    delete kept;
+    return result;
+  }
 }
