@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012-2021 Laboratoire de Recherche et Développement
+// Copyright (C) 2012-2022 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -40,6 +40,7 @@
 #include <spot/twaalgos/cobuchi.hh>
 #include <spot/twaalgos/cleanacc.hh>
 #include <spot/twaalgos/toparity.hh>
+#include <spot/twaalgos/zlktree.hh>
 
 namespace spot
 {
@@ -89,8 +90,10 @@ namespace spot
         wdba_minimize_ = opt->get("wdba-minimize", -1);
         gen_reduce_parity_ = opt->get("gen-reduce-parity", 1);
         simul_max_ = opt->get("simul-max", 4096);
+        merge_states_min_ = opt->get("merge-states-min", 128);
         wdba_det_max_ = opt->get("wdba-det-max", 4096);
         simul_trans_pruning_ = opt->get("simul-trans-pruning", 512);
+        acd_ = opt->get("acd", 1);
 
         if (sat_acc_ && sat_minimize_ == 0)
           sat_minimize_ = 1;        // Dicho.
@@ -118,6 +121,9 @@ namespace spot
   {
     if (opt == 0)
       return a;
+    if (merge_states_min_ > 0
+        && static_cast<unsigned>(merge_states_min_) < a->num_states())
+      a->merge_states();
     if (simul_max_ > 0 && static_cast<unsigned>(simul_max_) < a->num_states())
       return a;
 
@@ -233,10 +239,13 @@ namespace spot
   {
     if (PREF_ != Any && level_ != Low)
       tmp->remove_unused_ap();
-    if (COMP_)
+    bool was_complete = tmp->prop_complete().is_true();
+    if (COMP_ && !was_complete)
       tmp = complete(tmp);
     bool want_parity = type_ & Parity;
-    if (want_parity && tmp->acc().is_generalized_buchi())
+    if (want_parity && tmp->num_sets() > 1
+        && (tmp->acc().is_generalized_buchi()
+            || tmp->acc().is_generalized_co_buchi()))
       tmp = choose_degen(tmp);
     assert(!!SBACC_ == state_based_);
     if (state_based_)
@@ -245,7 +254,8 @@ namespace spot
       tmp = ensure_ba(tmp);
     if (want_parity)
       {
-        reduce_parity_here(tmp, COLORED_);
+        if (!acd_was_used_ || (COMP_ && !was_complete))
+          reduce_parity_here(tmp, COLORED_);
         parity_kind kind = parity_kind_any;
         parity_style style = parity_style_any;
         if ((type_ & ParityMin) == ParityMin)
@@ -290,6 +300,8 @@ namespace spot
     bool via_gba =
       (type_ == Buchi) || (type_ == GeneralizedBuchi) || (type_ == Monitor);
     bool want_parity = type_ & Parity;
+    acd_was_used_ = false;
+
     if (COLORED_ && !want_parity)
       throw std::runtime_error("postprocessor: the Colored setting only works "
                                "for parity acceptance");
@@ -335,18 +347,26 @@ namespace spot
         !(type_ == Generic && PREF_ == Any && level_ == Low))
       a = remove_alternation(a);
 
+    // If we do want a parity automaton, we can use to_parity().
+    // However (1) degeneralization is faster if the input is
+    // GBA, and (2) if we want a deterministic parity automaton and the
+    // input is not deterministic, that is useless here.  We need
+    // to determinize it first, and our deterministization
+    // function only deal with TGBA as input.
     if ((via_gba || (want_parity && !a->acc().is_parity()))
         && !a->acc().is_generalized_buchi())
       {
-        // If we do want a parity automaton, we can use to_parity().
-        // However (1) degeneralization is better if the input is
-        // GBA, and (2) if we want a deterministic parity automaton and the
-        // input is not deterministic, that is useless here.  We need
-        // to determinize it first, and our deterministization
-        // function only deal with TGBA as input.
         if (want_parity && (PREF_ != Deterministic || is_deterministic(a)))
           {
-            a = to_parity(a);
+            if (acd_)
+              {
+                a = acd_transform(a, COLORED_);
+                acd_was_used_ = true;
+              }
+            else
+              {
+                a = to_parity(a);
+              }
           }
         else
           {
@@ -402,10 +422,19 @@ namespace spot
 
     if (PREF_ == Any)
       {
-        if (type_ == Buchi)
-          a = choose_degen(a);
+        if (type_ == Buchi
+            || (type_ == CoBuchi && a->acc().is_generalized_co_buchi()))
+          {
+            a = choose_degen(a);
+          }
         else if (type_ == CoBuchi)
-          a = to_nca(a);
+          {
+            a = to_nca(a);
+            if (state_based_ && a->prop_state_acc().is_true())
+              a = do_sba_simul(a, simul_);
+            else
+              a = do_simul(a, simul_);
+          }
         return finalize(a);
       }
 
@@ -699,6 +728,8 @@ namespace spot
     if (type_ == CoBuchi)
       {
         unsigned ns = sim->num_states();
+        bool weak = sim->prop_weak().is_true();
+
         if (PREF_ == Deterministic)
           sim = to_dca(sim);
         else
@@ -706,8 +737,13 @@ namespace spot
 
         // if the input of to_dca/to_nca was weak, the number of
         // states has not changed, and running simulation is useless.
-        if (level_ != Low && ns < sim->num_states())
-          sim = do_simul(sim, simul_);
+        if (!weak || (level_ != Low && ns < sim->num_states()))
+          {
+            if (state_based_ && sim->prop_state_acc().is_true())
+              sim = do_sba_simul(sim, simul_);
+            else
+              sim = do_simul(sim, simul_);
+          }
       }
 
     return finalize(sim);
