@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012-2021 Laboratoire de Recherche et Développement
+// Copyright (C) 2012-2023 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -93,13 +93,7 @@ namespace spot
           return true;
         if (states > r.states)
           return false;
-
-        if (edges < r.edges)
-          return true;
-        if (edges > r.edges)
-          return false;
-
-        return false;
+        return edges < r.edges;
       }
 
       inline bool operator>(const automaton_size& r)
@@ -590,7 +584,7 @@ namespace spot
                 // C1 then (!C1)C2, instead of C1 then C2.
                 // With minatop_isop, we ensure that the no negative
                 // class variable will be seen (likewise for promises).
-                minato_isop isop(sig & one);
+                minato_isop isop(bdd_restrict(sig, one));
 
                 ++nb_minterms;
 
@@ -603,17 +597,12 @@ namespace spot
 
                     // Take the edge, and keep only the variable which
                     // are used to represent the class.
-                    bdd dst = bdd_existcomp(cond_acc_dest,
-                                             all_class_var_);
+                    bdd dst = bdd_existcomp(cond_acc_dest, all_class_var_);
 
                     // Keep only ones who are acceptance condition.
                     auto acc = bdd_to_mark(bdd_existcomp(cond_acc_dest,
                                                          all_proms_));
 
-                    // Keep the other!
-                    bdd cond = bdd_existcomp(cond_acc_dest,
-                                             sup_all_atomic_prop);
-
                     // Because we have complemented all the Inf
                     // acceptance conditions on the input automaton,
                     // we must revert them to create a new edge.
@@ -630,11 +619,11 @@ namespace spot
                             accst[srcst] = acc;
                             acc = {};
                           }
-                        gb->new_edge(dst.id(), src.id(), cond, acc);
+                        gb->new_edge(dst.id(), src.id(), one, acc);
                       }
                     else
                       {
-                        gb->new_edge(src.id(), dst.id(), cond, acc);
+                        gb->new_edge(src.id(), dst.id(), one, acc);
                       }
                   }
               }
