@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2013-2018, 2020-2021 Laboratoire de Recherche et
+// Copyright (C) 2013-2018, 2020-2022 Laboratoire de Recherche et
 // Développement de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -29,6 +29,7 @@
 #include <spot/twaalgos/product.hh>
 #include <spot/twaalgos/sccinfo.hh>
 #include <spot/twaalgos/hoa.hh>
+#include <spot/twaalgos/dbranch.hh>
 
 namespace spot
 {
@@ -61,15 +62,17 @@ namespace spot
         gf_guarantee_set_ = true;
       }
     ltl_split_ = opt->get("ltl-split", 1);
-    int tls_max_states = opt->get("tls-max-states", 64);
-    tls_max_states_ = std::max(0, tls_max_states);
+    tls_max_states_ = std::max(0, opt->get("tls-max-states", 64));
+    tls_max_ops_ = std::max(0, opt->get("tls-max-ops", 16));
     exprop_ = opt->get("exprop", -1);
+    branchpost_ = opt->get("branch-post", -1);
   }
 
   void translator::build_simplifier(const bdd_dict_ptr& dict)
   {
     tl_simplifier_options options(false, false, false);
     options.containment_max_states = tls_max_states_;
+    options.containment_max_ops = tls_max_ops_;
     switch (level_)
       {
       case High:
@@ -135,6 +138,9 @@ namespace spot
     twa_graph_ptr aut;
     twa_graph_ptr aut2 = nullptr;
 
+    bool split_hard =
+      type_ == Generic || (type_ & Parity) || type_ == GeneralizedBuchi;
+
     if (ltl_split_ && !r.is_syntactic_obligation())
       {
         formula r2 = r;
@@ -144,11 +150,11 @@ namespace spot
             r2 = r2[0];
             ++leading_x;
           }
-        if (type_ == Generic || type_ == GeneralizedBuchi)
+        if (split_hard)
           {
-            // F(q|u|f) = q|F(u)|F(f)  only for generic acceptance
+            // F(q|u|f) = q|F(u)|F(f)  disabled for GeneralizedBuchi
             // G(q&e&f) = q&G(e)&G(f)
-            bool want_u = r2.is({op::F, op::Or}) && (type_ == Generic);
+            bool want_u = r2.is({op::F, op::Or}) && (type_ != GeneralizedBuchi);
             if (want_u || r2.is({op::G, op::And}))
               {
                 std::vector<formula> susp;
@@ -211,20 +217,19 @@ namespace spot
             oblg.erase(i, oblg.end());
           }
 
+        // The only cases where we accept susp and rest to be both
+        // non-empty is when doing Generic/Parity/TGBA
         if (!susp.empty())
           {
-            // The only cases where we accept susp and rest to be both
-            // non-empty is when doing Generic acceptance or TGBA.
-            if (!rest.empty()
-                && !(type_ == Generic || type_ == GeneralizedBuchi))
+            if (!rest.empty() && !split_hard)
               {
                 rest.insert(rest.end(), susp.begin(), susp.end());
                 susp.clear();
               }
             // For Parity, we want to translate all suspendable
             // formulas at once.
-            if (rest.empty() && type_ & Parity)
-              susp = { formula::multop(r2.kind(), susp) };
+            //if (rest.empty() && type_ & Parity)
+            //  susp = { formula::multop(r2.kind(), susp) };
           }
         // For TGBA and BA, we only split if there is something to
         // suspend.
@@ -383,6 +388,7 @@ namespace spot
                      || type_ == GeneralizedBuchi)
                   aut2 = gf_guarantee_to_ba_maybe(r, simpl_->get_dict(),
                                                   det, state_based_);
+                acd_was_used_ = false;
                 if (aut2 && (pref_ & Deterministic))
                   return finalize(aut2);
                 if (!aut2 && (type_ == Generic
@@ -398,9 +404,22 @@ namespace spot
         bool exprop = unambiguous
           || (level_ == postprocessor::High && exprop_ != 0)
           || exprop_ > 0;
+        // branch-post: 1 == force branching postponement
+        //              0 == disable branching post. and delay_branching
+        //              2 == force delay_branching
+        //             -1 == auto (delay_branching)
+        // Some quick experiments suggests that branching postponement
+        // can produce larger automata on non-obligations formulas, and
+        // that even on obligation formulas, delay_branching is faster.
+        bool bpost = branchpost_ == 1;
         aut = ltl_to_tgba_fm(r, simpl_->get_dict(), exprop,
-                             true, false, false, nullptr, nullptr,
+                             true, bpost, false, nullptr, nullptr,
                              unambiguous);
+        if (!bpost && branchpost_ != 0 && delay_branching_here(aut))
+          {
+            aut->purge_unreachable_states();
+            aut->merge_edges();
+          }
       }
 
     aut = this->postprocessor::run(aut, r);
