@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2014-2016, 2018 Laboratoire de Recherche et
+// Copyright (C) 2014-2016, 2018, 2019 Laboratoire de Recherche et
 // Développement de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -37,7 +37,7 @@
 static void
 syntax(char* prog)
 {
-  std::cerr << prog << " file" << std::endl;
+  std::cerr << prog << " file\n";
   exit(2);
 }
 
