@@ -1,5 +1,5 @@
 # -*- mode: python; coding: utf-8 -*-
-# Copyright (C) 2017, 2021 Laboratoire de Recherche et
+# Copyright (C) 2017, 2021, 2022 Laboratoire de Recherche et
 # Développement de l'Epita
 #
 # This file is part of Spot, a model checking library.
@@ -18,6 +18,8 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import spot
+from unittest import TestCase
+tc = TestCase()
 
 aut = spot.translate('(Ga -> Gb) W c')
 si = spot.scc_info(aut)
@@ -26,10 +28,10 @@ si = spot.scc_info(aut)
 # if the generation of the automaton changes, so just scan
 # for it.
 rej = [j for j in range(si.scc_count()) if si.is_rejecting_scc(j)]
-assert len(rej) == 1
+tc.assertEqual(len(rej), 1)
 s = spot.decompose_scc(si, rej[0]).to_str('hoa', '1.1')
 
-assert (s == """HOA: v1.1
+tc.assertEqual(s, """HOA: v1.1
 States: 3
 Start: 0
 AP: 3 "b" "a" "c"
@@ -56,7 +58,8 @@ except RuntimeError:
 else:
     raise AssertionError
 
-assert (spot.decompose_scc(si, 0, True).to_str('hoa', '1.1') == """HOA: v1.1
+tc.assertEqual(spot.decompose_scc(si, 0, True).to_str('hoa', '1.1'),
+"""HOA: v1.1
 States: 4
 Start: 0
 AP: 3 "b" "a" "c"
@@ -81,7 +84,8 @@ State: 3
 [1] 3
 --END--""")
 
-assert (spot.decompose_scc(si, 2, True).to_str('hoa', '1.1') == """HOA: v1.1
+tc.assertEqual(spot.decompose_scc(si, 2, True).to_str('hoa', '1.1'),
+"""HOA: v1.1
 States: 2
 Start: 0
 AP: 3 "b" "a" "c"
@@ -103,4 +107,4 @@ try:
 except RuntimeError:
     pass
 else:
-    raise AssertionError
+    raise AssertionError("missing exception")
